# Generated by Django 4.1.8 on 2023-04-12 12:56

import uuid

import django.core.validators
import django.db.models.deletion
import django_fsm
import wbcore.contrib.directory.models.entries
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("geography", "0001_initial"),
        ("currency", "0001_initial"),
        ("agenda", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="CompanyType",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("title", models.CharField(max_length=128, unique=True, verbose_name="Title")),
                (
                    "slugify_title",
                    models.CharField(
                        blank=True, max_length=128, null=True, unique=True, verbose_name="Slugified Title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Company Type",
                "verbose_name_plural": "Company Types",
            },
        ),
        migrations.CreateModel(
            name="CustomerStatus",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("title", models.CharField(max_length=32, unique=True, verbose_name="Title")),
                (
                    "slugify_title",
                    models.CharField(
                        blank=True, max_length=32, null=True, unique=True, verbose_name="Slugified Title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Status",
                "verbose_name_plural": "Customer Statuses",
            },
        ),
        migrations.CreateModel(
            name="Entry",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("is_active", models.BooleanField(default=True)),
                ("deletion_datetime", models.DateTimeField(blank=True, null=True)),
                ("uuid", models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                (
                    "entry_type",
                    models.CharField(
                        choices=[("PERSON", "Person"), ("COMPANY", "Company")],
                        default="",
                        max_length=255,
                        verbose_name="Type",
                    ),
                ),
                (
                    "additional_fields",
                    models.JSONField(blank=True, default=dict, null=True, verbose_name="Additional Fields"),
                ),
                (
                    "activity_heat",
                    models.FloatField(
                        default=0,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(1),
                        ],
                        verbose_name="Activity Heat",
                    ),
                ),
                (
                    "slugify_computed_str",
                    models.CharField(blank=True, max_length=512, null=True, verbose_name="Slugified Search Name"),
                ),
                (
                    "profile_image",
                    models.ImageField(
                        blank=True,
                        max_length=256,
                        null=True,
                        upload_to=wbcore.contrib.directory.models.entries.upload_to_profile_images,
                    ),
                ),
                (
                    "signature",
                    models.ImageField(
                        blank=True,
                        max_length=256,
                        null=True,
                        upload_to=wbcore.contrib.directory.models.entries.upload_to_signatures,
                    ),
                ),
                ("external_identfier", models.CharField(blank=True, max_length=128, null=True)),
            ],
            options={
                "verbose_name": "Entry",
                "verbose_name_plural": "Entries",
                "ordering": ("entry_type",),
            },
        ),
        migrations.CreateModel(
            name="Position",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("title", models.CharField(max_length=128, unique=True, verbose_name="Title")),
                (
                    "slugify_title",
                    models.CharField(
                        blank=True, max_length=128, null=True, unique=True, verbose_name="Slugified Title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Position",
                "verbose_name_plural": "Positions",
            },
        ),
        migrations.CreateModel(
            name="Specialization",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("title", models.CharField(max_length=128, unique=True, verbose_name="Title")),
                (
                    "slugify_title",
                    models.CharField(
                        blank=True, max_length=128, null=True, unique=True, verbose_name="Slugified Title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Specialization",
                "verbose_name_plural": "Specializations",
            },
        ),
        migrations.CreateModel(
            name="Company",
            fields=[
                (
                    "entry_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="directory.entry",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Name")),
                (
                    "headcount",
                    models.CharField(blank=True, default="", max_length=32, verbose_name="Number of employees"),
                ),
                (
                    "tier",
                    models.CharField(
                        blank=True,
                        choices=[("ONE", "1"), ("TWO", "2"), ("THREE", "3"), ("FOUR", "4"), ("FIVE", "5")],
                        max_length=16,
                        null=True,
                        verbose_name="Tier",
                    ),
                ),
                (
                    "asset_allocation",
                    models.CharField(blank=True, default="", max_length=255, verbose_name="Asset Allocation"),
                ),
                (
                    "geographic_focus",
                    models.CharField(blank=True, default="", max_length=255, verbose_name="Geographic Focus"),
                ),
                (
                    "assets_under_management",
                    models.CharField(blank=True, default="", max_length=255, verbose_name="AUM"),
                ),
                (
                    "discretionairy_advisory",
                    models.FloatField(default=0.0, verbose_name="Discretionary Advisory"),
                ),
                ("description", models.TextField(blank=True, default="", verbose_name="Description")),
            ],
            options={
                "verbose_name": "Company",
                "verbose_name_plural": "Companies",
            },
            bases=("directory.entry",),
        ),
        migrations.CreateModel(
            name="Person",
            fields=[
                (
                    "entry_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="directory.entry",
                    ),
                ),
                (
                    "prefix",
                    models.CharField(
                        choices=[
                            ("MR", "Mr."),
                            ("MRS", "Mrs."),
                            ("PROF", "Prof."),
                            ("DR", "Dr."),
                            ("MED", "Med."),
                            ("ME", "Me."),
                        ],
                        max_length=4,
                        verbose_name="Prefix",
                    ),
                ),
                ("first_name", models.CharField(max_length=255, verbose_name="First Name")),
                ("last_name", models.CharField(max_length=255, verbose_name="Last Name")),
                ("initials", models.CharField(blank=True, max_length=4, null=True, verbose_name="Initials")),
                ("active_employee", models.BooleanField(default=True, verbose_name="Available Employee")),
                ("birthday", models.DateField(blank=True, null=True, verbose_name="Birthday")),
                (
                    "personality_profile_red",
                    models.FloatField(
                        default=0,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(1),
                        ],
                    ),
                ),
                (
                    "personality_profile_green",
                    models.FloatField(
                        default=0,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(1),
                        ],
                    ),
                ),
                (
                    "personality_profile_blue",
                    models.FloatField(
                        default=0,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(1),
                        ],
                    ),
                ),
                ("formal", models.BooleanField(default=True, verbose_name="Formal")),
            ],
            options={
                "verbose_name": "Person",
                "verbose_name_plural": "Persons",
            },
            bases=("directory.entry",),
        ),
        migrations.CreateModel(
            name="WebsiteContact",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "location",
                    models.CharField(
                        choices=[
                            ("WORK", "Work"),
                            ("PRIVATE", "Private"),
                            ("HOME", "Home"),
                            ("OTHER", "Other"),
                        ],
                        default="WORK",
                        max_length=32,
                        verbose_name="Location",
                    ),
                ),
                ("url", models.URLField(verbose_name="URL")),
                (
                    "entry",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="websites",
                        to="directory.entry",
                        verbose_name="Entry",
                    ),
                ),
            ],
            options={
                "verbose_name": "Website Contact",
                "verbose_name_plural": "Website Contacts",
            },
        ),
        migrations.CreateModel(
            name="TelephoneContact",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "location",
                    models.CharField(
                        choices=[
                            ("WORK", "Work"),
                            ("PRIVATE", "Private"),
                            ("HOME", "Home"),
                            ("OTHER", "Other"),
                        ],
                        default="WORK",
                        max_length=32,
                        verbose_name="Location",
                    ),
                ),
                ("number", models.CharField(max_length=255, verbose_name="Number")),
                (
                    "telephone_type",
                    models.CharField(
                        choices=[("FIX", "Fix"), ("CELL", "Cell"), ("FAX", "Fax"), ("OTHER", "Other")],
                        default="FIX",
                        max_length=32,
                        verbose_name="Type",
                    ),
                ),
                (
                    "entry",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="telephones",
                        to="directory.entry",
                        verbose_name="Entry",
                    ),
                ),
            ],
            options={
                "verbose_name": "Telephone Contact",
                "verbose_name_plural": "Telephone Contacts",
            },
        ),
        migrations.CreateModel(
            name="SocialMediaContact",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "location",
                    models.CharField(
                        choices=[
                            ("WORK", "Work"),
                            ("PRIVATE", "Private"),
                            ("HOME", "Home"),
                            ("OTHER", "Other"),
                        ],
                        default="WORK",
                        max_length=32,
                        verbose_name="Location",
                    ),
                ),
                (
                    "platform",
                    models.CharField(
                        choices=[
                            ("FACEBOOK", "Facebook"),
                            ("INSTAGRAM", "Instagram"),
                            ("LINKEDIN", "LinkedIn"),
                            ("REDDIT", "Reddit"),
                            ("TIKTOK", "TikTok"),
                            ("TUMBLR", "Tumblr"),
                            ("TWITTER", "Twitter"),
                            ("XING", "Xing"),
                        ],
                        help_text="The social media platform for this contact",
                        max_length=32,
                        verbose_name="Platform",
                    ),
                ),
                ("url", models.URLField(verbose_name="URL")),
                (
                    "entry",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="social_media",
                        to="directory.entry",
                        verbose_name="Entry",
                    ),
                ),
            ],
            options={
                "verbose_name": "Social Media Contact",
                "verbose_name_plural": "Social Media Contacts",
            },
        ),
        migrations.CreateModel(
            name="RelationshipType",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("title", models.CharField(max_length=256, unique=True, verbose_name="Relationship Type")),
                (
                    "slugify_title",
                    models.CharField(
                        blank=True, max_length=256, null=True, unique=True, verbose_name="Slugified Title"
                    ),
                ),
                (
                    "counter_relationship",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="reverse_counter_relationship",
                        to="directory.relationshiptype",
                        verbose_name="Counter Relationship",
                    ),
                ),
            ],
            options={
                "verbose_name": "Relationship Type",
                "verbose_name_plural": "Relationship Types",
            },
        ),
        migrations.CreateModel(
            name="Relationship",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                (
                    "from_entry",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="from_entry",
                        to="directory.entry",
                        verbose_name="Relationship From",
                    ),
                ),
                (
                    "relationship_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="directory.relationshiptype",
                        verbose_name="Type",
                    ),
                ),
                (
                    "to_entry",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="to_entry",
                        to="directory.entry",
                        verbose_name="Relationship To",
                    ),
                ),
            ],
            options={
                "verbose_name": "Relationship",
                "verbose_name_plural": "Relationships",
            },
        ),
        migrations.AddField(
            model_name="entry",
            name="relationships",
            field=models.ManyToManyField(
                blank=True,
                through="directory.Relationship",
                to="directory.entry",
                verbose_name="The Entry's Relationships",
            ),
        ),
        migrations.CreateModel(
            name="EmployerEmployeeRelationship",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "position",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="position_of_employee",
                        to="directory.position",
                        verbose_name="Position",
                    ),
                ),
            ],
            options={
                "verbose_name": "Employer Employee Relationship",
            },
        ),
        migrations.CreateModel(
            name="EmailContact",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "location",
                    models.CharField(
                        choices=[
                            ("WORK", "Work"),
                            ("PRIVATE", "Private"),
                            ("HOME", "Home"),
                            ("OTHER", "Other"),
                        ],
                        default="WORK",
                        max_length=32,
                        verbose_name="Location",
                    ),
                ),
                ("address", models.EmailField(max_length=254, verbose_name="Email Address")),
                (
                    "entry",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="emails",
                        to="directory.entry",
                        verbose_name="Entry",
                    ),
                ),
            ],
            options={
                "verbose_name": "E-Mail Contact",
                "verbose_name_plural": "E-Mail Contacts",
            },
        ),
        migrations.CreateModel(
            name="ClientManagerRelationship",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "status",
                    django_fsm.FSMField(
                        choices=[
                            ("DRAFT", "Draft"),
                            ("PENDINGADD", "Pending Add"),
                            ("PENDINGREMOVE", "Pending Remove"),
                            ("APPROVED", "Approved"),
                            ("REMOVED", "Removed"),
                        ],
                        default="DRAFT",
                        max_length=50,
                        verbose_name="Status",
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                (
                    "client",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="client_of",
                        to="directory.entry",
                        verbose_name="Client",
                    ),
                ),
            ],
            options={
                "verbose_name": "Client Manager Relationship",
                "verbose_name_plural": "Client Manager Relationships",
            },
        ),
        migrations.CreateModel(
            name="BankingContact",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                (
                    "status",
                    django_fsm.FSMField(
                        choices=[("DRAFT", "Draft"), ("PENDING", "Pending"), ("APPROVED", "Approved")],
                        default="DRAFT",
                        max_length=50,
                        verbose_name="Status",
                    ),
                ),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "location",
                    models.CharField(
                        choices=[
                            ("WORK", "Work"),
                            ("PRIVATE", "Private"),
                            ("HOME", "Home"),
                            ("OTHER", "Other"),
                        ],
                        default="WORK",
                        max_length=32,
                        verbose_name="Location",
                    ),
                ),
                ("institute", models.CharField(max_length=255, verbose_name="Institute")),
                (
                    "institute_additional",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Institute (Additional)"),
                ),
                ("iban", models.CharField(blank=True, max_length=38, null=True, verbose_name="IBAN")),
                (
                    "swift_bic",
                    models.CharField(blank=True, max_length=11, null=True, verbose_name="SWIFT/BIC"),
                ),
                ("edited", models.DateTimeField(auto_now=True, verbose_name="Edited")),
                (
                    "additional_information",
                    models.TextField(
                        blank=True,
                        help_text="Can be used in place of IBAN to communicate banking information",
                        null=True,
                        verbose_name="Additional Information",
                    ),
                ),
                (
                    "currency",
                    models.ForeignKey(
                        blank=True,
                        db_column="linked_currency",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="currency.currency",
                        verbose_name="Currency",
                    ),
                ),
                (
                    "entry",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="banking",
                        to="directory.entry",
                        verbose_name="Entry",
                    ),
                ),
            ],
            options={
                "verbose_name": "Banking Contact",
                "verbose_name_plural": "Banking Contacts",
                "permissions": (("can_approve_bankingcontact", "Can approve Banking Contact"),),
            },
        ),
        migrations.CreateModel(
            name="AddressContact",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("primary", models.BooleanField(default=False, verbose_name="Primary")),
                (
                    "location",
                    models.CharField(
                        choices=[
                            ("WORK", "Work"),
                            ("PRIVATE", "Private"),
                            ("HOME", "Home"),
                            ("OTHER", "Other"),
                        ],
                        default="WORK",
                        max_length=32,
                        verbose_name="Location",
                    ),
                ),
                ("street", models.CharField(blank=True, max_length=255, null=True, verbose_name="Street")),
                (
                    "street_additional",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Street (Additional)"),
                ),
                ("zip", models.CharField(blank=True, max_length=255, null=True, verbose_name="ZIP")),
                ("city", models.CharField(blank=True, max_length=255, null=True, verbose_name="City")),
                ("province", models.CharField(blank=True, max_length=255, null=True, verbose_name="Province")),
                (
                    "entry",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="addresses",
                        to="directory.entry",
                        verbose_name="Entry",
                    ),
                ),
                (
                    "geography_city",
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to={"level": 3},
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="contact_city",
                        to="geography.geography",
                        verbose_name="City",
                    ),
                ),
            ],
            options={
                "verbose_name": "Address Contact",
                "verbose_name_plural": "Address Contacts",
            },
        ),
        migrations.AddConstraint(
            model_name="websitecontact",
            constraint=models.UniqueConstraint(fields=("url", "entry"), name="unique_website_entry"),
        ),
        migrations.AddConstraint(
            model_name="telephonecontact",
            constraint=models.UniqueConstraint(fields=("number", "entry"), name="unique_number_entry"),
        ),
        migrations.AddConstraint(
            model_name="socialmediacontact",
            constraint=models.UniqueConstraint(fields=("url", "entry"), name="unique_social_entry"),
        ),
        migrations.AddField(
            model_name="person",
            name="employers",
            field=models.ManyToManyField(
                blank=True,
                help_text="The person's employers",
                related_name="employees",
                through="directory.EmployerEmployeeRelationship",
                to="directory.company",
                verbose_name="Employers",
            ),
        ),
        migrations.AddField(
            model_name="person",
            name="specializations",
            field=models.ManyToManyField(blank=True, to="directory.specialization", verbose_name="Specializations"),
        ),
        migrations.AddField(
            model_name="entry",
            name="relationship_managers",
            field=models.ManyToManyField(
                blank=True,
                help_text="People in charge of this entry",
                related_name="clients",
                through="directory.ClientManagerRelationship",
                to="directory.person",
                verbose_name="Relationship Managers",
            ),
        ),
        migrations.AddField(
            model_name="employeremployeerelationship",
            name="employee",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="employer",
                to="directory.person",
                verbose_name="Employee",
            ),
        ),
        migrations.AddField(
            model_name="employeremployeerelationship",
            name="employer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="employee",
                to="directory.company",
                verbose_name="Employer",
            ),
        ),
        migrations.AddConstraint(
            model_name="emailcontact",
            constraint=models.UniqueConstraint(fields=("entry", "address"), name="entry_address_unique_together"),
        ),
        migrations.AddField(
            model_name="company",
            name="customer_status",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="company",
                to="directory.customerstatus",
                verbose_name="Customer Status",
            ),
        ),
        migrations.AddField(
            model_name="company",
            name="type",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="company",
                to="directory.companytype",
                verbose_name="Type",
            ),
        ),
        migrations.AddField(
            model_name="clientmanagerrelationship",
            name="relationship_manager",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="manager_of",
                to="directory.person",
                verbose_name="Relationship Manager",
            ),
        ),
        migrations.AddConstraint(
            model_name="bankingcontact",
            constraint=models.UniqueConstraint(fields=("iban", "entry"), name="unique_iban_entry"),
        ),
        migrations.AddConstraint(
            model_name="employeremployeerelationship",
            constraint=models.UniqueConstraint(fields=("employee", "employer"), name="unique_employeremployee"),
        ),
    ]



from django.apps import AppConfig
from django.db import DEFAULT_DB_ALIAS
from django.db.models.signals import post_migrate


def notification_type_post_migrate(app_config, using=DEFAULT_DB_ALIAS, **kwargs):
    from django.contrib.contenttypes.models import ContentType

    from wbcore.contrib.notifications.models import NotificationType

    ContentType.objects.clear_cache()
    # For each model in an app, generate all notification types listed in the meta class of each model
    for klass in app_config.get_models():
        # Get the contenttype and set all notification types for this content type to stale
        # All "touched" notification types will be automatically set to stale=False
        contenttype = ContentType.objects.db_manager(using).get_for_model(klass, for_concrete_model=False)  # type: ignore
        NotificationType.objects.db_manager(using).filter(contenttype=contenttype).update(stale=True)

        for code, title, help_text, web, mobile, email, resource_button_label, is_lock in getattr(
            klass._meta, "notification_types", []
        ):
            NotificationType.objects.using(using).update_or_create(
                code=code,
                defaults={
                    "contenttype": contenttype,
                    "title": title,
                    "help_text": help_text,
                    "default_enable_web": web,
                    "default_enable_mobile": mobile,
                    "default_enable_email": email,
                    "resource_button_label": resource_button_label,
                    "stale": False,
                    "is_lock": is_lock,
                },
            )


class NotificationsConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "wbcore.contrib.notifications"

    def ready(self):
        post_migrate.connect(notification_type_post_migrate)



# Generated by Django 4.1.8 on 2023-04-17 07:22

import django.db.models.deletion
from django.db import migrations, models


def migrate_script(apps, schema_editor):
    User = apps.get_model("authentication", "User")
    from wbcore.contrib.directory.models import Person

    for user in User.objects.filter(profile__isnull=True):
        user.profile_id = Person.get_or_create_with_user(user).id
        user.save()


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0002_auto_20230414_1553"),
        ("authentication", "0002_profile"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="profile",
            field=models.OneToOneField(
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="user_account",
                to="directory.person",
            ),
            preserve_default=False,
        ),
    ]



from django.forms import ValidationError
from django.utils import timezone

from wbcore import serializers
from wbcore.contrib.example_app.models import Season
from wbcore.contrib.example_app.serializers import (
    LeagueRepresentationSerializer,
    PlayerRepresentationSerializer,
    TeamRepresentationSerializer,
)


class SeasonModelSerializer(serializers.ModelSerializer):
    _league = LeagueRepresentationSerializer(source="league", many=False)
    _winner = TeamRepresentationSerializer(source="winner", many=False)
    _top_scorer = PlayerRepresentationSerializer(source="top_scorer")

    def validate(self, data):
        date_range = data.get("date_range", None)
        winner = data.get("winner", None)
        top_scorer = data.get("top_scorer", None)

        if date_range and date_range.upper > timezone.now().date():
            if winner:
                raise ValidationError({"winner": "You cannot choose a winner, when the season is not finished"})
            if top_scorer:
                raise ValidationError(
                    {"top_scorer": "You cannot choose a top scorer, when the season is not finished"}
                )

        return super().validate(data)

    class Meta:
        model = Season
        fields = [
            "id",
            "name",
            "league",
            "_league",
            "date_range",
            "file",
            "winner",
            "_winner",
            "top_scorer",
            "_top_scorer",
        ]


class SeasonRepresentationSerializer(serializers.RepresentationSerializer):
    _detail = serializers.HyperlinkField(reverse_name="example_app:season-detail")

    class Meta:
        model = Season
        fields = ("id", "name", "_detail")



from django.utils.translation import gettext as _
from rest_framework.exceptions import ValidationError

from wbcore import serializers as wb_serializers
from wbcore.contrib.workflow.models import Data, DataValue, ProcessStep, Workflow
from wbcore.contrib.workflow.serializers.workflow import (
    WorkflowRepresentationSerializer,
)


class DataModelSerializer(wb_serializers.ModelSerializer):
    workflow = wb_serializers.PrimaryKeyRelatedField(
        default=wb_serializers.DefaultFromGET("workflow"),
        queryset=Workflow.objects.all(),
        label=_("Workflow"),
    )
    _workflow = WorkflowRepresentationSerializer(source="workflow")

    def validate(self, data: dict) -> dict:
        label: str | None = data.get("label", self.instance.label if self.instance else None)
        workflow: Workflow | None = data.get("workflow", self.instance.workflow if self.instance else None)
        required: bool | None = data.get("required", self.instance.required if self.instance else None)
        data_type: Data.DataType | None = data.get("data_type", self.instance.data_type if self.instance else None)
        if "default" in data:
            default: str = data["default"]
        else:
            default: str | None = self.instance.default if self.instance else None

        if label and hasattr(ProcessStep, label):
            raise ValidationError({"label": _("A field with this name already exists in the process step model.")})

        duplicate_data_obj = Data.objects.filter(label=label, workflow=workflow)
        if self.instance:
            duplicate_data_obj = duplicate_data_obj.exclude(id=self.instance.id)
        if duplicate_data_obj.exists():
            raise ValidationError(
                {"label": _("A data object with this label was already created for the selected workflow.")}
            )

        if default:
            if required:
                raise ValidationError({"required": _("Cannot set both required and a default value.")})
            if data_type:
                try:
                    Data.cast_value_to_datatype(data_type, default)
                except ValueError:
                    if data_type == Data.DataType.DATE:
                        raise ValidationError(
                            {
                                "default": _(
                                    "Invalid default value for this data type. Please use a date formatted to 'day.month.year'."
                                )
                            }
                        )
                    elif data_type == Data.DataType.DATETIME:
                        raise ValidationError(
                            {
                                "default": _(
                                    "Invalid default value for this data type. Please use a datetime formatted to 'day.month.year hour:minute:second' in the 24h format."
                                )
                            }
                        )
                    raise ValidationError({"default": _("Invalid default value for this data type.")})

        return data

    class Meta:
        model = Data
        fields = (
            "id",
            "workflow",
            "_workflow",
            "label",
            "help_text",
            "data_type",
            "required",
            "default",
        )


class DataRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:workflow:data-detail")

    class Meta:
        model = Data
        fields = (
            "id",
            "_detail",
        )


class DataValueModelSerializer(wb_serializers.ModelSerializer):
    _data = DataRepresentationSerializer(source="data")

    def validate(self, data: dict) -> dict:
        value: str | None = data.get("value", self.instance.value if self.instance else None)
        data_obj: Data | None = data.get("data", self.instance.data if self.instance else None)

        if data_obj and value:
            try:
                Data.cast_value_to_datatype(data_obj.data_type, value)
            except ValueError:
                if data_obj.data_type == Data.DataType.DATE:
                    raise ValidationError(
                        {
                            "value": _(
                                "Invalid value for this data type. Please use a date formatted to 'day.month.year'."
                            )
                        }
                    )
                elif data_obj.data_type == Data.DataType.DATETIME:
                    raise ValidationError(
                        {
                            "value": _(
                                "Invalid value for this data type. Please use a datetime formatted to 'day.month.year hour:minute:second' in the 24h format."
                            )
                        }
                    )
                raise ValidationError({"value": _("Invalid value for this data type.")})

        return data

    class Meta:
        model = DataValue
        fields = (
            "id",
            "data",
            "_data",
            "value",
        )


class DataValueRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = DataValue
        fields = ("id",)



import enum
from contextlib import suppress
from types import DynamicClassAttribute

from django.conf import settings
from django.db.models import TextChoices
from django.db.models.enums import ChoicesMeta

from wbcore.utils.importlib import import_from_dotted_path

DEFAULT_ICON_BACKEND = "wbcore.contrib.icons.backends.material.IconBackend"
FALLBACK_ICON_VALUE = "FALLBACK_ICON"


class WBIconMeta(ChoicesMeta):
    """A metaclass for creating a enum choices."""

    def __new__(metacls, classname, bases, classdict, **kwds):
        dict.__setitem__(
            classdict, FALLBACK_ICON_VALUE, (FALLBACK_ICON_VALUE, "Fallback Icon")
        )  # add a default member that represent the fallback in case it's not yet implement in the imported backend
        cls = super().__new__(metacls, classname, bases, classdict, **kwds)
        with suppress(ModuleNotFoundError):
            setattr(
                cls,
                "icon_backend",
                import_from_dotted_path(getattr(settings, "WBCORE_ICON_BACKEND", DEFAULT_ICON_BACKEND)),
            )
            icon_backend = cls.icon_backend
            # For each enumeration values, attached an "icon" property to its members
            for member, value in zip(cls.__members__.values(), cls.values):
                member._icon_ = getattr(icon_backend, value, icon_backend.fallback_icon)
        return enum.unique(cls)


class WBIcon(TextChoices, metaclass=WBIconMeta):
    """
    This enumeration defines the supported icons. For each attribute, we expect an equivalent attribute in each of the Icon Backends.

    The actual value of the icon can be retreived using `WBIcon.MY_ICON.icon`

    Procedure to add new icon:
    1) Add the enumeration member in this class (e.g. NEW_ICON = "NEW_ICON", "My new Icon")
    2) For each supported icon backend, create the equivalent attribute pointing to the material icon value (e.g NEW_ICON = "my_new_icon_in_google_material_icons")
    """

    @DynamicClassAttribute
    def icon(self):
        return self._icon_

    ACCOUNT_BALANCE = "ACCOUNT_BALANCE", "Account Balance"
    ADD = "ADD", "Add"
    APPROVE = "APPROVE", "Approve"
    BANK = "BANK", "Bank"
    BIRTHDAY = "BIRTHDAY", "Birthday"
    BOOKMARK = "BOOKMARK", "Bookmark"
    CALENDAR = "CALENDAR", "Calendar"
    CHART_AREA = "CHART_AREA", "Chart area"
    CHART_BARS_HORIZONTAL = "CHART_BARS_HORIZONTAL", "Chart bars horizonal"
    CHART_BARS_VERTICAL = "CHART_BARS_VERTICAL", "Chart bars vertical"
    CHART_BARS_VERTICAL_INCREASING = "CHART_BARS_VERTICAL_INCREASING", "Chart bars vertical increasing"
    CHART_LINE = "CHART_LINE", "Chart line"
    CHART_PIE = "CHART_PIE", "Chart pie"
    CHART_PYRAMID = "CHART_PYRAMID", "Chart piramid"
    CHART_SWITCHES = "CHART_SWITCHES", "Chart switches"
    CHART_WATERFALL = "CHART_WATERFALL", "Chart waterfall"
    CHART_MULTILINE = "CHART_MULTILINE", "Chart multiline"
    CLIPBOARD = "CLIPBOARD", "Clipboard"
    CONFIRM = "CONFIRM", "Confirm"
    CONFIGURE = "CONFIGURE", "configure"
    COPY = "COPY", "Copy"
    CURRENCY_EXCHANGE = "CURRENCY_EXCHANGE", "Currency Exchange"
    DASHBOARD = "DASHBOARD", "Dashboard"
    DATA_GRID = "DATA_GRID", "Data grid"
    DATA_LIST = "DATA_LIST", "Data list"
    DATA_EXPLORATION = "DATA_EXPLORATION", "Data exploration"
    DAY_OFF = "DAY_OFF", "Day off"
    DEAL = "DEAL", "Deal"
    DEAL_MONEY = "DEAL_MONEY", "Deal money"
    DECISION_STEP = "DECISION_STEP", "Decision step"
    DELETE = "DELETE", "Delete"
    DENY = "DENY", "Deny"
    DISABLED = "DISABLED", "Disabled"
    DOCUMENT = "DOCUMENT", "Document"
    DOCUMENT_IN_PROGRESS = "DOCUMENT_IN_PROGRESS", "Document in progress"
    DOCUMENT_PRIVATE = "DOCUMENT_PRIVATE", "Document private"
    DOCUMENT_WITH_DOLLAR = "DOCUMENT_WITH_DOLLAR", "Document with Dollar"
    DOCUMENT_WITH_ATTACHMENT = "DOCUMENT_WITH_ATTACHMENT", "Document with Attachment"
    DOLLAR = "DOLLAR", "Dollar"
    DOWNLOAD = "DOWNLOAD", "Download"
    EDIT = "EDIT", "Edit"
    END = "END", "End"
    ENUMERATION = "ENUMERATION", "Enumeration"
    EURO = "EURO", "Euro"
    EVENT = "EVENT", "Event"
    EVENT_ACCEPTED = "EVENT_ACCEPTED", "Event accepted"
    EXPEND = "EXPEND", "Expend"
    FAVORITE = "FAVORITE", "Favorite"
    FEEDBACK = "FEEDBACK", "Feedback"
    FOLDERS_ADD = "FOLDERS_ADD", "Folders add"
    FOLDERS_MONEY = "FOLDERS_MONEY", "Folders money"
    FOLDERS_OPEN = "FOLDERS_OPEN", "Folders open"
    GENERATE_NEXT = "GENERATE_NEXT", "Generate next"
    GOAL = "GOAL", "Goal"
    GROUPS = "GROUPS", "Groups"
    HISTORY = "HISTORY", "History"
    HOME = "HOME", "Home"
    IGNORE = "IGNORE", "Ignore"
    IMPORT_EXPORT = "IMPORT_EXPORT", "Import Export"
    INFO = "INFO", "Info"
    INVERSE = "INVERSE", "Inverse"
    JOIN_STEP = "JOIN_STEP", "Join Step"
    LINK = "LINK", "Link"
    LOCATION = "LOCATION", "Location"
    LOCK = "LOCK", "Lock"
    LUNCH = "LUNCH", "Lunch"
    MAIL = "MAIL", "Mail"
    MAIL_MONEY = "MAIL_MONEY", "Mail Money"
    MAIL_OPEN = "MAIL_OPEN", "Mail open"
    MAIL_POSTED = "MAIL_POSTED", "Mail posted"
    MERGE = "MERGE", "Merge"
    NEWSPAPER = "NEWSPAPER", "Newspaper"
    NEXT = "NEXT", "Next"
    NOTEBOOK = "NOTEBOOK", "Notebook"
    OTHER = "OTHER", "Other"
    PEOPLE = "PEOPLE", "People"
    PENDING = "PENDING", "Pending"
    PERSON = "PERSON", "Person"
    PERSON_ADD = "PERSON_ADD", "Person Add"
    PHONE = "PHONE", "Phone"
    PHONE_ADD = "PHONE_ADD", "Phone Add"
    PREVIOUS = "PREVIOUS", "Previous"
    PUBLIC = "PUBLIC", "Public"
    QUESTION = "QUESTION", "QUESTION"
    REDO = "REDO", "Redo"
    REFRESH = "REFRESH", "Refresh"
    REGENERATE = "REGENERATE", "Regenerate"
    REJECT = "REJECT", "Reject"
    REPEAT = "REPEAT", "Repeat"
    REPLACE = "REPLACE", "Replace"
    REVIEW = "REVIEW", "Review"
    RUNNING = "RUNNING", "Running"
    SAVE = "SAVE", "Save"
    SCHEDULE = "SCHEDULE", "Schedule"
    SCRIPT = "SCRIPT", "Script"
    SEND = "SEND", "Send"
    SEND_LATER = "SEND_LATER", "Send Later"
    SETTINGS = "SETTINGS", "Settings"
    SHARE = "SHARE", "Share"
    SHRINK = "SHRINK", "Shrink"
    SICK_LEAVE = "SICK_LEAVE", "Sick leave"
    SPLIT = "SPLIT", "Split"
    START = "START", "Start"
    STATS = "STATS", "Stats"
    SUPERVISE = "SUPERVISE", "Supervise"
    SYNCHRONIZE = "SYNCHRONIZE", "Synchronize"
    TABLE = "TABLE", "Table"
    TIME_UP = "TIME_UP", "Time Up"
    TRADE = "TRADE", "Trade"
    TRAINING = "TRAINING", "Training"
    UNDO = "UNDO", "Undo"
    UNFILTER = "UNFILTER", "Unfilter"
    UNFOLD = "UNFOLD", "Unfold"
    UNLINK = "UNLINK", "Unlink"
    UNLOCK = "UNLOCK", "Unlock"
    UPLOAD = "UPLOAD", "Upload"
    VACATION = "VACATION", "Vacation"
    VIEW = "VIEW", "View"
    WARNING = "WARNING", "Warning"
    WORK = "WORK", "Work"



# Generated by Django 4.1.8 on 2023-04-11 12:08

import uuid

import django.db.models.deletion
import mptt.fields
import wbcore.contrib.documents.models.documents
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Document",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                (
                    "permission_type",
                    models.CharField(
                        choices=[("INTERNAL", "Internal"), ("PUBLIC", "Public"), ("PRIVATE", "Private")],
                        default="PRIVATE",
                        max_length=8,
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("description", models.TextField(blank=True, default="", verbose_name="Description")),
                (
                    "file",
                    models.FileField(
                        max_length=256,
                        upload_to=wbcore.contrib.documents.models.documents.upload_to,
                        verbose_name="File",
                    ),
                ),
                ("system_created", models.BooleanField(default=False, verbose_name="System Created")),
                (
                    "system_key",
                    models.CharField(blank=True, max_length=255, null=True, unique=True, verbose_name="System Key"),
                ),
                ("created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("updated", models.DateTimeField(auto_now=True, null=True, verbose_name="Updated")),
                (
                    "creator",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="created_%(app_label)s_%(class)s",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Document",
                "verbose_name_plural": "Documents",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "administrate"),
            },
        ),
        migrations.CreateModel(
            name="ShareableLink",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("valid_until", models.DateTimeField(blank=True, null=True, verbose_name="Valid Until")),
                ("one_time_link", models.BooleanField(default=False, verbose_name="One Time Link")),
                ("manual_invalid", models.BooleanField(default=False)),
                ("uuid", models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                (
                    "document",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="shareable_links",
                        to="documents.document",
                    ),
                ),
            ],
            options={
                "verbose_name": "Shareable Link",
                "verbose_name_plural": "Shareable Links",
            },
        ),
        migrations.CreateModel(
            name="ShareableLinkAccess",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("metadata", models.JSONField(default=dict)),
                ("accessed", models.DateTimeField(auto_now_add=True)),
                (
                    "shareable_link",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="accesses",
                        to="documents.shareablelink",
                    ),
                ),
            ],
            options={
                "verbose_name": "Shareable Link Access",
                "verbose_name_plural": "Shareable Link Accesses",
            },
        ),
        migrations.CreateModel(
            name="DocumentType",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("name", models.CharField(max_length=255)),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                ("tree_id", models.PositiveIntegerField(db_index=True, editable=False)),
                ("level", models.PositiveIntegerField(editable=False)),
                (
                    "parent",
                    mptt.fields.TreeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="documents.documenttype",
                        verbose_name="Parent",
                    ),
                ),
            ],
            options={
                "verbose_name": "Document Type",
                "verbose_name_plural": "Document Types",
            },
        ),
        migrations.CreateModel(
            name="DocumentModelRelationship",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("object_id", models.PositiveIntegerField()),
                (
                    "content_type",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"),
                ),
                (
                    "document",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="relationships",
                        to="documents.document",
                    ),
                ),
            ],
            options={
                "verbose_name": "Document Model Relationship",
                "verbose_name_plural": "Document Model Relationships",
            },
        ),
        migrations.AddField(
            model_name="document",
            name="document_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="documents",
                to="documents.documenttype",
                verbose_name="Document Type",
            ),
        ),
        migrations.AddIndex(
            model_name="documentmodelrelationship",
            index=models.Index(fields=["content_type", "object_id"], name="documents_d_content_bd3456_idx"),
        ),
    ]



class Templates:
    TEMPLATES = [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "NAME": "django",
            "DIRS": [],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.debug",
                    "django.template.context_processors.request",
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ],
            },
        },
    ]



import pytest
from rest_framework.exceptions import ValidationError

from wbcore.serializers import BooleanField
from wbcore.serializers.fields.types import WBCoreType


class TestBooleanField:
    def setup_method(self):
        self.field = BooleanField()

    def test_not_none(self):
        assert self.field is not None

    @pytest.mark.parametrize(
        "input, expected",
        [
            (True, True),
            (False, False),
            (1, True),
            (0, False),
            ("1", True),
            ("0", False),
        ],
    )
    def test_to_internal_value(self, input, expected):
        assert self.field.to_internal_value(input) == expected

    @pytest.mark.parametrize("input", ["", [], {}, None])
    def test_to_internal_value_validation_error(self, input):
        with pytest.raises(ValidationError):
            self.field.to_internal_value(input)

    def test_field_type(self):
        assert self.field.field_type == WBCoreType.BOOLEAN.value

    def test_representation(self):
        assert self.field.get_representation(None, "field_name") == (
            "field_name",
            {
                "key": "field_name",
                "label": None,
                "type": self.field.field_type,
                "required": True,
                "read_only": False,
                "decorators": [],
                "depends_on": [],
            },
        )



from typing import Iterable

from django.contrib.messages.storage.base import BaseStorage, Message
from django.contrib.messages.storage.fallback import FallbackStorage
from rest_framework_simplejwt.settings import api_settings

MESSAGE_TYPE_MAPPING = {
    10: "debug",
    20: "info",
    25: "success",
    30: "warning",
    40: "error",
}


class InMemoryMessageStorage(BaseStorage):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._messages = []

    def _get(self, *args, **kwargs):
        """
        Retrieve a list of messages. Returns a tuple of (messages, all_retrieved).
        """
        return self._messages, True

    def _store(self, messages, *args, **kwargs):
        """
        Store a list of messages.
        """
        self._messages = messages

    def add(self, level, message, extra_tags="", *args, **kwargs):
        """
        Adds a message to the storage.
        """
        if not message:
            return
        message_instance = Message(level, message, extra_tags=extra_tags)
        self._messages.append(message_instance)

    def __iter__(self):
        return iter(self._messages)

    def serialize_messages(self) -> Iterable[dict[str, str | int]]:
        for message in self._messages:
            extra_parameters = dict([param.split("=") for param in message.extra_tags.split(" ") if param != ""])
            auto_close = int(extra_parameters.get("auto_close", "5")) * 1000
            yield {
                "message": message.message,
                "type": MESSAGE_TYPE_MAPPING[message.level],
                "auto_close": auto_close,
            }


def route_message_storage(request):
    # This is a simple hack to detect when the request is initiated by our frontend client or by a session based page (e.g. Admin)
    if request.META.get(api_settings.AUTH_HEADER_NAME):
        return InMemoryMessageStorage(request)
    return FallbackStorage(request)



from django.utils.translation import gettext as _
from rest_framework.reverse import reverse

from wbcore.contrib.icons import WBIcon
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig


class PlayerButtonConfig(ButtonViewConfig):
    def get_custom_instance_buttons(self) -> set:
        if player_id := self.view.kwargs.get("pk"):
            return {
                bt.WidgetButton(
                    label=_("Statistics"),
                    icon=WBIcon.TABLE.icon,
                    endpoint=reverse("example_app:player-statistics-list", args=[player_id], request=self.request),
                ),
            }
        return set()



from datetime import date, timedelta
from typing import Optional

from django.utils.translation import gettext as _

from wbcore.contrib.color.enums import WBColor
from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.view_config import DisplayViewConfig

from ...models import UserActivity


class UserActivityModelDisplay(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="IP", label=_("IP")),
                dp.Field(key="user", label=_("User")),
                dp.Field(key="date", label=_("First Login")),
                dp.Field(key="latest_refresh", label=_("Latest Refresh")),
                dp.Field(key="time_online_minute", label=_("Time Online (Minutes)")),
                dp.Field(key="user_agent_info", label=_("Agent Info")),
            ],
            formatting=[
                dp.Formatting(
                    column="status",
                    formatting_rules=[
                        dp.FormattingRule(
                            style={"backgroundColor": WBColor.GREEN_LIGHT.value},
                            condition=("==", UserActivity.SUCCESS),
                        ),
                        dp.FormattingRule(
                            style={"backgroundColor": WBColor.RED_LIGHT.value},
                            condition=("==", UserActivity.FAILED),
                        ),
                    ],
                )
            ],
            legends=[
                dp.Legend(
                    key="status",
                    items=[
                        dp.LegendItem(
                            icon=WBColor.GREEN_LIGHT.value,
                            label=UserActivity.SUCCESS,
                            value=UserActivity.SUCCESS,
                        ),
                        dp.LegendItem(
                            icon=WBColor.RED_LIGHT.value,
                            label=UserActivity.FAILED,
                            value=UserActivity.FAILED,
                        ),
                    ],
                )
            ],
        )


class UserActivityModelUserDisplay(UserActivityModelDisplay):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="IP", label=_("IP")),
                dp.Field(key="user", label=_("User")),
                dp.Field(key="date", label=_("First Login")),
                dp.Field(key="latest_refresh", label=_("Latest Refresh")),
                dp.Field(key="time_online_minute", label=_("Time Online (Minutes)")),
                dp.Field(key="user_agent_info", label=_("Agent Info")),
            ],
            formatting=super().get_list_display().formatting,
            legends=super().get_list_display().legends,
        )


class UserActivityTableDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        today = date.today()
        display = dp.ListDisplay(
            fields=[
                dp.Field(key="user_repr", label=_("User")),
                dp.Field(key="beforeyesterday_activity", label=f"{today - timedelta(days=2)}"),
                dp.Field(key="yesterday_activity", label=f"{today - timedelta(days=1)}"),
                dp.Field(key="today_activity", label=f"{today}"),
            ]
        )
        return display



from copy import copy

from django.conf import settings
from django.urls import path
from django.views.generic import TemplateView


class FrontendView(TemplateView):
    template_name = settings.FRONTEND_TEMPLATE

    def get_context_data(self, **kwargs):
        context = {**super().get_context_data(**kwargs), **copy(settings.FRONTEND_CONTEXT)}
        if version := self.request.GET.get("frontend_version", None):
            base_url = f"{settings.CDN_BASE_ENDPOINT_URL}/{version}/"
            context["JS_RUNTIME_URL"] = f"{base_url}runtime.js"
            context["JS_URL"] = f"{base_url}main.js"
            context["CSS_URL"] = f"{base_url}main.css"
        context["WBCORE_CONTEXT"] = context
        return context

    @classmethod
    def bundled_view(cls, url_path):
        return path(route=url_path, view=cls.as_view(), name="frontend")



# Generated by Django 4.1 on 2023-02-23 12:05

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="NotificationType",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("code", models.CharField(max_length=128, unique=True)),
                ("title", models.CharField(default="", max_length=128)),
                ("help_text", models.CharField(default="", max_length=512)),
                ("stale", models.BooleanField(default=False)),
                ("default_enable_web", models.BooleanField(default=False)),
                ("default_enable_mobile", models.BooleanField(default=False)),
                ("default_enable_email", models.BooleanField(default=False)),
                (
                    "contenttype",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            options={
                "verbose_name": "Notification Type",
                "verbose_name_plural": "Notification Types",
            },
        ),
        migrations.CreateModel(
            name="NotificationUserToken",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("token", models.CharField(max_length=256)),
                ("device_type", models.CharField(choices=[("WEB", "Web"), ("MOBILE", "Mobile")], max_length=16)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notifications_tokens",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="NotificationTypeSetting",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("enable_web", models.BooleanField(default=False)),
                ("enable_mobile", models.BooleanField(default=False)),
                ("enable_email", models.BooleanField(default=False)),
                (
                    "notification_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="user_settings",
                        to="notifications.notificationtype",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="wbnotification_user_settings",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Notification",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255)),
                ("body", models.TextField(default="")),
                ("endpoint", models.CharField(blank=True, max_length=255, null=True)),
                ("sent", models.DateTimeField(blank=True, null=True)),
                ("read", models.DateTimeField(blank=True, null=True)),
                (
                    "notification_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notifications",
                        to="notifications.notificationtype",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notifications_notifications",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Notification",
                "verbose_name_plural": "Notifications",
            },
        ),
    ]



from django.utils.translation import gettext as _

from wbcore.menus import Menu, MenuItem

TEAM_MENUITEM = MenuItem(
    label=_("Team"),
    endpoint="example_app:team-list",
    add=MenuItem(
        label=_("Create Team"),
        endpoint="example_app:team-list",
    ),
)
LEAGUE_MENUITEM = MenuItem(
    label=_("League"),
    endpoint="example_app:league-list",
    add=MenuItem(
        label=_("Create League"),
        endpoint="example_app:league-list",
    ),
)
STADIUM_MENUITEM = MenuItem(
    label=_("Stadium"),
    endpoint="example_app:stadium-list",
    add=MenuItem(
        label=_("Create Stadium"),
        endpoint="example_app:stadium-list",
    ),
)
FIGURE_MENUITEM = MenuItem(
    label=_("Figure"),
    endpoint="example_app:figure-list",
    add=MenuItem(
        label=_("Create Figure"),
        endpoint="example_app:figure-list",
    ),
)
SPORT_MENUITEM = MenuItem(
    label=_("Sport"),
    endpoint="example_app:sport-list",
    add=MenuItem(
        label=_("Create Sport"),
        endpoint="example_app:sport-list",
    ),
)
PLAYER_MENUITEM = MenuItem(
    label=_("Player"),
    endpoint="example_app:player-list",
    add=MenuItem(
        label=_("Create player"),
        endpoint="example_app:player-list",
    ),
)

EXAMPLE_APP_MENU = Menu(
    label="Sports App",
    items=[
        FIGURE_MENUITEM,
        PLAYER_MENUITEM,
        TEAM_MENUITEM,
        STADIUM_MENUITEM,
        LEAGUE_MENUITEM,
        SPORT_MENUITEM,
    ],
)



from configurations import values
from rest_framework.reverse import reverse


class AgendaConfigurationMixin:
    FRONTEND_MENU_CALENDAR = "wbcore.contrib.agenda.configurations.frontend_menu_calendar"
    DEFAULT_CREATE_ENDPOINT_BASENAME = values.Value("wbcore:agenda:calendaritem-list", environ_prefix=None)


def frontend_menu_calendar(request):
    return reverse("wbcore:agenda:owncalendar-list", request=request)



from django.contrib.auth import get_user_model
from django.db import models
from django.db.models.query import QuerySet

from wbcore.contrib.notifications.models.notification_types import (
    NotificationTypeSetting,
)


class NotificationUserTokenManager(models.Manager):
    def filter_for_user_settings(self, user_setting: NotificationTypeSetting) -> QuerySet["NotificationUserToken"]:
        device_type = NotificationUserToken.NotificationDeviceType
        device_types = []

        if user_setting.enable_web:
            device_types.append(device_type.WEB)

        if user_setting.enable_mobile:
            device_types.append(device_type.MOBILE)

        return self.filter(
            user=user_setting.user,
            device_type__in=device_types,
        )


class NotificationUserToken(models.Model):
    class NotificationDeviceType(models.TextChoices):
        WEB = "WEB", "Web"
        MOBILE = "MOBILE", "Mobile"

    user = models.ForeignKey(to=get_user_model(), related_name="notifications_tokens", on_delete=models.CASCADE)
    token = models.CharField(max_length=256)
    device_type = models.CharField(max_length=16, choices=NotificationDeviceType.choices)

    updated = models.DateTimeField(auto_now=True)

    objects = NotificationUserTokenManager()

    def __str__(self) -> str:
        return f"{self.user}: {self.token} ({self.device_type})"

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=["user", "token", "device_type"], name="unique_user_token_device")
        ]



from django.contrib.auth.models import Permission


def perm_to_permission(perm: str) -> Permission:
    """
    Convert a identifier string permission format in 'app_label.codename'
    (teremd as *perm*) to a django permission instance.

    Args:
        perm: The string permission identifier in the form 'app_label.codename'

    Returns:
        The Permission object corresponding to the given string identifier

    Raises:
        AttributeError: When the given string identifier does not match the expected format 'app_label.codename'
    """
    try:
        app_label, codename = perm.split(".", 1)
    except IndexError:
        raise AttributeError(
            "The format of identifier string permission (perm) is wrong. " "It should be in 'app_label.codename'."
        )
    else:
        permission = Permission.objects.get(content_type__app_label=app_label, codename=codename)
        return permission



import pytest
from django.test import Client, TestCase
from django.urls import reverse
from wbcore.contrib.authentication.factories import SuperUserFactory
from wbcore.contrib.example_app.factories import TeamFactory
from wbcore.contrib.example_app.models import Team
from wbcore.contrib.example_app.serializers import TeamModelSerializer
from wbcore.contrib.example_app.tests.test_viewsets.test_utils_viewsets import (  # get_update_view,
    find_instances_in_response,
    get_create_view,
    get_delete_view,
    get_detail_view,
    get_partial_view,
    get_update_view,
)
from wbcore.contrib.example_app.viewsets import TeamModelViewSet


@pytest.mark.django_db
class TestTeamModelViewSet(TestCase):
    def setUp(self):
        self.user = SuperUserFactory()
        self.client = Client()
        self.client.force_login(user=self.user)
        self.list_url = reverse("example_app:team-list")
        self.detail_url_str = "example_app:team-detail"

    def test_list_view(self):
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, 200)

    def test_create_view(self):
        team = TeamFactory()
        response = get_create_view(self.client, team, self.user, self.list_url, TeamModelViewSet)
        self.assertEqual(response.status_code, 201)
        self.assertTrue(Team.objects.filter(name=team.name).exists())

    def test_detail_view(self):
        team = TeamFactory()
        response = get_detail_view(self.client, team.pk, self.detail_url_str)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["instance"]["name"], team.name)

    def test_update_view(self):
        instance = TeamFactory()
        instance.name = "Updated Instance"
        response = get_update_view(self.client, instance, TeamModelSerializer, self.detail_url_str)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["instance"]["name"], instance.name)

    def test_partial_update_view(self):
        instance = TeamFactory()
        response = get_partial_view(self.client, instance.id, {"name": "Updated Instance"}, self.detail_url_str)
        instance.refresh_from_db()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["instance"]["name"], instance.name)

    def test_delete_view(self):
        team = TeamFactory()
        response = get_delete_view(self.client, self.detail_url_str, team.pk)
        self.assertEqual(response.status_code, 204)
        self.assertFalse(Team.objects.filter(pk=team.pk).exists())

    def test_ordering_fields(self):
        team_a = TeamFactory(name="BBB", order=1)
        team_b = TeamFactory(name="AAA", order=0)
        team_c = TeamFactory(name="CCC", order=2)

        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["count"], Team.objects.count())
        self.assertEqual(response.data["results"][0]["id"], team_b.id)
        self.assertEqual(response.data["results"][1]["id"], team_a.id)
        self.assertEqual(response.data["results"][2]["id"], team_c.id)

    def test_team_stadium(self):
        team_a = TeamFactory()
        team_b = TeamFactory(home_stadium=team_a.home_stadium)
        team_c = TeamFactory()
        expected_number_of_teams = Team.objects.filter(home_stadium=team_a.home_stadium).count()
        team_stadium_url = reverse("example_app:team-stadium-list", args=[team_a.home_stadium.id])
        response = self.client.get(team_stadium_url)
        team_a_found, team_b_found, team_c_found = find_instances_in_response([team_a, team_b, team_c], response)
        self.assertEqual(
            response.data["count"],
            expected_number_of_teams,
            f"The answer should contain {expected_number_of_teams} teams",
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue(team_a_found, "Team A was not found in Response")
        self.assertTrue(team_b_found, "Team B was not found in Response")
        self.assertFalse(team_c_found, "Team C was found in Response, but should not be found")



from contextlib import suppress
from datetime import datetime

from celery import shared_task
from django.contrib.contenttypes.models import ContentType
from django.db import models, transaction
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _
from wbcore.contrib.authentication.models.users import User
from wbcore.contrib.guardian.utils import reload_permissions
from wbcore.permissions.mixins import PermissionMixin
from wbcore.utils.itertools import get_inheriting_subclasses


class PermissionObjectModelMixin(PermissionMixin):
    """
    Permission object Mixin that set the default behavior for permission assignments.
    Methods can be supercharged in order to customize behavior.
    """

    id: int

    class PermissionType(models.TextChoices):
        INTERNAL = "INTERNAL", _("Internal")
        PUBLIC = "PUBLIC", _("Public")
        PRIVATE = "PRIVATE", _("Private")

    guardian_shortcut_kwargs = {"accept_global_perms": False}

    permission_type = models.CharField(default=PermissionType.PRIVATE, choices=PermissionType.choices, max_length=8)
    creator = models.ForeignKey(
        User,
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        related_name="created_%(app_label)s_%(class)s",
    )

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        content_type = ContentType.objects.get_for_model(self)
        transaction.on_commit(lambda: assign_user_permissions_for_object_as_task.delay(content_type.id, self.id))  # type: ignore

    class Meta:
        abstract = True
        default_permissions = ("add", "change", "delete", "view", "administrate")

    def get_permissions_for_user(self, user: User, created: datetime | None = None) -> dict[str, bool]:
        """
        Determine the permissions for a given user on this object. This method uses a best guess approach.
        If this approach is not suitable for your needs, please override this method in your subclass and
        avoid a call to `super().get_permissions_for_user`.

        This method checks the user's status and permissions to determine what actions
        they are allowed to perform on the object. The permissions are returned as a
        dictionary where the keys are permission strings and the values are booleans
        indicating whether the permission is editable.

        Behavior:
            - If the user is a superuser, they are granted all permissions (non-editable).
            - If the user is internal and has admin permissions, they are granted all permissions (non-editable).
            - For other internal users, permissions are checked individually and added to the
              result if the user has them (all set as non-editable).
            - Non-internal users receive no permissions.
        """

        # Superuser have access regardless, but we do not want to overpolute the permission table
        if user.is_superuser:
            return {}

        # If the user is internal and has the admin permission, this user can do anything
        if user.is_internal and user.has_perm(self.admin_perm_str):
            return {
                self.view_perm_str: False,
                self.change_perm_str: False,
                self.delete_perm_str: False,
                self.select_perm_str: False,
                self.add_perm_str: False,
            }

        permissions = {}
        if user.is_internal:
            if user.has_perm(self.view_perm_str):
                permissions[self.view_perm_str] = False

            if user.has_perm(self.change_perm_str):
                permissions[self.change_perm_str] = False

            if user.has_perm(self.delete_perm_str):
                permissions[self.delete_perm_str] = False

            if user.has_perm(self.select_perm_str):
                permissions[self.select_perm_str] = False

            if user.has_perm(self.add_perm_str):
                permissions[self.add_perm_str] = False

        return permissions

    def reload_permissions(self, prune_existing: bool = True, force_pruning: bool = False):
        reload_permissions(
            self.__class__.objects, instance=self, prune_existing=prune_existing, force_pruning=force_pruning
        )


@receiver(post_save, sender=User)
def post_save_user(sender, instance, created, **kwargs):
    """
    Triggers the load_permission_objects as a celery task for all connected signals on User save
    """
    if created:
        transaction.on_commit(lambda: assign_object_permissions_for_user_as_task.delay(instance.id))  # type: ignore


@shared_task
def assign_user_permissions_for_object_as_task(
    content_type_id: int, instance_id: int, prune_existing: bool | None = True
):
    """
    Utility function to create object permission from obj decoupling from the main thread
    """
    content_type = ContentType.objects.get(id=content_type_id)
    if model_class := content_type.model_class():
        with suppress(model_class.DoesNotExist):
            instance = model_class.objects.get(id=instance_id)
            instance.reload_permissions(prune_existing=prune_existing)  # type: ignore


@shared_task
def assign_object_permissions_for_user_as_task(user_id: int):
    """
    Utility function to create object permission from user decoupling from the main thread
    """

    user = User.objects.get(id=user_id)
    for permission_class in get_inheriting_subclasses(PermissionObjectModelMixin):
        reload_permissions(permission_class.objects, user=user, prune_existing=True)



from datetime import date
from enum import Enum

from django.forms import ValidationError
from django.utils import timezone
from django.utils.translation import gettext as _
from rest_framework.reverse import reverse

from wbcore import serializers
from wbcore.contrib.example_app.models import League, Sport
from wbcore.contrib.example_app.serializers import (
    SportPersonRepresentationSerializer,
    SportRepresentationSerializer,
)
from wbcore.contrib.example_app.utils import get_event_types_for_league
from wbcore.contrib.geography.serializers import GeographyRepresentationSerializer


class LeagueErrorMessages(Enum):
    league_exists = _("A league with this name already exists for this type of sport.")
    wrong_established_date = _("The founding date must be in the past.")


class LeagueModelSerializer(serializers.ModelSerializer):
    _commissioner = SportPersonRepresentationSerializer(source="commissioner")
    _country = GeographyRepresentationSerializer(filter_params={"level": 1}, source="country")
    _sport = SportRepresentationSerializer(source="sport")

    def validate(self, data: dict) -> dict:
        name: str | None = data.get("name", self.instance.name if self.instance else None)
        sport: Sport | None = data.get("sport", self.instance.sport if self.instance else None)
        established_date: date | None = data.get("established_date", None)
        if name and sport:
            league = League.objects.filter(name=name, sport=sport)
            if obj := self.instance:
                league = league.exclude(id=obj.id)
            if league.exists():
                raise ValidationError({"name": LeagueErrorMessages.league_exists.value})
        if established_date and established_date > timezone.now().date():
            raise ValidationError({"established_date": LeagueErrorMessages.wrong_established_date.value})
        return super().validate(data)

    @serializers.register_only_instance_resource()
    def matches_inline(self, instance: League, request, user, view, **kwargs) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        return {
            "recent_matches": reverse(
                "example_app:match-league-list", args=[instance.pk], request=self.context["request"]
            )
        }

    @serializers.register_only_instance_resource()
    def table_inline(self, instance: League, request, user, view, **kwargs) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        return {
            "table": reverse(
                "example_app:teamresults-league-list", args=[instance.pk], request=self.context["request"]
            )
        }

    @serializers.register_only_instance_resource()
    def league_statistics_inline(self, instance: League, request, user, view, **kwargs) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        statistics_inlines = {}
        for event_type in get_event_types_for_league(instance.pk):
            statistics_inlines.update(
                {
                    f"player_{event_type['slugified_name']}": reverse(
                        "example_app:league-player-statistics-list",
                        args=[instance.pk, event_type["id"]],
                        request=self.context["request"],
                    ),
                    f"team_{event_type['slugified_name']}": reverse(
                        "example_app:league-team-statistics-list",
                        args=[instance.pk, event_type["id"]],
                        request=self.context["request"],
                    ),
                },
            )

        return statistics_inlines

    class Meta:
        model = League
        fields = (
            "id",
            "name",
            "sport",
            "_sport",
            "country",
            "_country",
            "points_per_win",
            "points_per_draw",
            "points_per_loss",
            "established_date",
            "commissioner",
            "_commissioner",
            "website",
            "_additional_resources",
        )
        optional_fields = ("teams",)


class LeagueRepresentationSerializer(serializers.RepresentationSerializer):
    _detail = serializers.HyperlinkField(reverse_name="example_app:league-detail")

    class Meta:
        model = League
        fields = ("id", "name", "computed_str", "_detail")



import factory

from wbcore.contrib.example_app.models import Player, SportPerson


class SportPersonFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SportPerson

    profile = factory.SubFactory("wbcore.contrib.directory.factories.PersonFactory")
    profile_image = factory.django.ImageField(filename="image_profile.jpeg")
    first_name = factory.SelfAttribute("profile.first_name")
    last_name = factory.SelfAttribute("profile.last_name")

    @factory.post_generation
    def roles(self, create, extracted, **kwargs):
        if extracted:
            for role in extracted:
                self.roles.add(role)


class PlayerFactory(SportPersonFactory):
    position = factory.Faker("text", max_nb_chars=15)
    current_team = factory.SubFactory("wbcore.contrib.example_app.factories.TeamFactory")

    @factory.post_generation
    def former_teams(self, create, extracted, **kwargs):
        if extracted:
            for team in extracted:
                self.former_teams.add(team)

    class Meta:
        model = Player



# Generated by Django 5.0.4 on 2024-05-06 08:57

import picklefield.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("io", "0005_exportsource_data_alter_exportsource_query_str_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="exportsource",
            name="query_params",
            field=picklefield.fields.PickledObjectField(
                blank=True, editable=False, verbose_name="SQL query parameters to be used with the sql query"
            ),
        ),
    ]



import pytest
from django.forms.models import model_to_dict
from rest_framework.test import APITestCase
from wbcore.contrib.example_app.factories import SportFactory
from wbcore.contrib.example_app.serializers import SportModelSerializer


@pytest.mark.django_db
class TestSportModelSerializer(APITestCase):
    def test_sport_serializer(self):
        sport_data: dict = model_to_dict(SportFactory.build())
        sport_serializer = SportModelSerializer(data=sport_data)

        self.assertTrue(sport_serializer.is_valid())



from django.utils.translation import gettext_lazy as _

from wbcore import filters as wb_filters
from wbcore.contrib.workflow.models import Condition, Transition


class ConditionFilter(wb_filters.FilterSet):
    transition = wb_filters.ModelMultipleChoiceFilter(
        label=_("Transitions"),
        queryset=Transition.objects.all(),
        endpoint=Transition.get_representation_endpoint(),
        value_key=Transition.get_representation_value_key(),
        label_key=Transition.get_representation_label_key(),
    )

    operator = wb_filters.MultipleChoiceFilter(choices=Condition.Operator.choices, label=_("Operators"))

    class Meta:
        model = Condition
        fields = {
            "attribute_name": ["exact", "icontains"],
            "expected_value": ["exact", "icontains"],
            "negate_operator": ["exact"],
        }



from django.utils.translation import gettext as _

from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.list_display import ListDisplay
from wbcore.metadata.configs.display.view_config import DisplayViewConfig

TEAMRESULTS_FIELDS = [
    dp.Field(key="league", label=_("League")),
    dp.Field(key="team", label=_("Team")),
    dp.Field(key="games_played", label=_("Games")),
    dp.Field(key="wins", label=_("Wins")),
    dp.Field(key="draws", label=_("Draws")),
    dp.Field(key="losses", label=_("Losses")),
    dp.Field(key="match_points_for", label=_("Match Points For")),
    dp.Field(key="match_points_against", label=_("Match Points Against")),
    dp.Field(key="match_point_difference", label=_("Match Point Difference")),
    dp.Field(key="points", label=_("Points")),
    dp.Field(key="form", label=_("Form")),
]


class TeamResultsDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> ListDisplay:
        return dp.ListDisplay(fields=TEAMRESULTS_FIELDS)


class TeamResultsLeagueDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> ListDisplay:
        fields = TEAMRESULTS_FIELDS.copy()
        fields.pop(0)
        return dp.ListDisplay(fields=fields)



from typing import Optional

from django.utils.translation import gettext as _

from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
)
from wbcore.metadata.configs.display.instance_display.utils import repeat_field
from wbcore.metadata.configs.display.view_config import DisplayViewConfig


class BuildingDisplay(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="name", label=_("Name")),
                dp.Field(key="address", label=_("Address")),
            ]
        )

    def get_instance_display(self) -> Display:
        return create_simple_display([["name", "address"]])


class ConferenceRoomDisplay(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="name", label=_("Name")),
                dp.Field(key="email", label=_("E-Mail-Address")),
                dp.Field(key="building", label=_("Building")),
                dp.Field(key="capacity", label=_("Capacity")),
                dp.Field(key="is_videoconference_capable", label=_("Capable of Videoconferencing")),
            ]
        )

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [[repeat_field(2, "name"), "email"], ["building", "capacity", "is_videoconference_capable"]]
        )



from django.utils.translation import gettext as _

from wbcore.metadata.configs.titles import TitleViewConfig


class ShareableLinkModelTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Shareable Links")

    def get_instance_title(self):
        return _("Shareable Link")

    def get_create_title(self):
        return _("New Shareable Link")



from dataclasses import dataclass, field
from enum import Enum
from typing import Iterable, Optional

# Type Definitions
GridArea = list[list[str]]
SerializedField = dict[str, str | bool]
SerializedCardLayout = dict[str, str | SerializedField | list[SerializedField]]
SerializedTable = dict[str, str | bool]
SerializedChart = dict[str, str]
SerializedSection = list
SerializedLayout = dict[str, str | SerializedSection | SerializedTable]
SerializedItem = dict[str, str | dict[str, SerializedLayout]]


class DisplayType(Enum):
    FORM = "form"
    TABLE = "table"
    CHART = "chart"
    CALENDAR = "calendar"


class ItemType(Enum):
    PAGE = "page"
    TAB = "tab"


class InvalidOperatorError(Exception):
    pass


class Operator(Enum):
    S = "<"
    SE = "<="
    G = ">"
    GE = ">="
    E = "=="

    @classmethod
    def get_operator(cls, op: str) -> "Operator":
        operator_dict = {o.value: o for o in cls}
        try:
            return operator_dict[op]
        except KeyError:
            raise InvalidOperatorError(f"`{op}` is not a valid operator")


def fr(fractions: int) -> str:
    return f"{fractions}fr"


def px(pixels: int) -> str:
    return f"{pixels}px"


def pct(percent: float) -> str:
    return f"{percent}%"


def repeat(times: int, unit: str) -> str:
    return f"repeat({times}, {unit})"


def grid_definition(*units):
    return " ".join(units)


def layout_size(width: Optional[int] = None, operator: str | Operator = Operator.S) -> str:
    if width is None:
        return "default"

    if isinstance(operator, str):
        operator = Operator.get_operator(operator)

    return f"{operator.value}{width}"


@dataclass
class Table:
    key: str
    editable: bool = False

    def serialize(self) -> SerializedTable:
        return {
            "key": self.key,
            "editable": self.editable,
        }


@dataclass
class Chart:
    key: str

    def serialize(self) -> SerializedChart:
        return {"key": self.key}


@dataclass
class Layout:  # This has to be renamed to FormLayout
    grid_areas: GridArea
    grid_columns: Optional[str] = None
    grid_rows: Optional[str] = None
    sections: list = field(default_factory=list)
    tables: list[Table] = field(default_factory=list)
    charts: list[Chart] = field(default_factory=list)

    def serialize(self) -> SerializedLayout:
        serialized_layout = {
            "gridTemplateAreas": " ".join([f"'{' '.join(row)}'" for row in self.grid_areas]),
            "sections": self.sections,
            "tables": [table.serialize() for table in self.tables],
            "charts": [chart.serialize() for chart in self.charts],
        }

        if self.grid_columns:
            serialized_layout["gridTemplateColumns"] = self.grid_columns

        if self.grid_rows:
            serialized_layout["gridTemplateRows"] = self.grid_rows

        return serialized_layout


@dataclass
class Field:
    key: str
    show_label: bool = True
    label: str | None = None

    def serialize(self) -> SerializedField:
        serialized_field = {"key": self.key, "show_label": self.show_label}

        if self.label and self.show_label:
            serialized_field["label"] = self.label

        return serialized_field


# @dataclass
# class TableLayout:
#     def serialize(self) -> SerializedTableLayout:
#         pass


@dataclass
class CardLayout:
    title: Field
    fields: list[Field] = field(default_factory=list)

    color: str | None = None
    background_color: str | None = None
    image: str | None = None

    def serialize(self) -> SerializedCardLayout:
        serialized_layout = {"title": self.title.serialize(), "fields": [field.serialize() for field in self.fields]}

        if self.color:
            serialized_layout["color"] = self.color

        if self.background_color:
            serialized_layout["background_color"] = self.background_color

        if self.image:
            serialized_layout["image"] = self.image

        return serialized_layout


@dataclass
class Item:
    layouts: dict[str, Layout]
    title: str = ""

    def serialize(self) -> SerializedItem:
        return {"title": self.title, "layout": {key: layout.serialize() for key, layout in self.layouts.items()}}


@dataclass
class Display:
    display_type: DisplayType
    items: Iterable[Item]  # Consider to also be able to just a layout to make it redundant to use an Item here
    item_type: ItemType = ItemType.PAGE  # This can maybe also be nullable?

    def serialize(self) -> dict[str, str | list[SerializedItem]]:
        return {
            "display_type": self.display_type.value,
            "item_type": self.item_type.value,
            "items": [item.serialize() for item in self.items],
        }

    def __iter__(self):
        for item in self.items:
            yield item.serialize()


if __name__ == "__main__":
    # display = Display(
    #     display_type=DisplayType.FORM,
    #     items=[
    #         Item(
    #             title="First Page",
    #             layouts={
    #                 layout_size(): Layout(
    #                     grid_columns=repeat(6, fr(1)),
    #                     grid_areas=[["status", "status", "status", "title", "period", "period"]],
    #                 ),
    #                 layout_size(400): Layout(
    #                     grid_columns=repeat(3, fr(1)),
    #                     grid_areas=[["status", "status", "status"], ["title", "period", "period"]],
    #                 ),
    #             },
    #         )
    #     ],
    # )
    # display = Display(
    #     display_type=DisplayType.TABLE,
    #     items=[
    #         Item(
    #             layouts={
    #                 layout_size(): Layout(
    #                     grid_columns=repeat(4, fr(1)),
    #                     grid_areas=[
    #                         ["period", "status", "title", "other_field"],
    #                         ["period", "status", "title", "other_field1"],
    #                     ],
    #                 )
    #             }
    #         )
    #     ],
    # )
    # print(display.serialize())
    # print(list(display) == display.serialize())

    print(layout_size(500, Operator.E))  # noqa: T201



from ..configurations import configuration_registry

CompanyModelSerializer = configuration_registry.company_model_serializer
CompanyRepresentationSerializer = configuration_registry.company_representation_serializer
PersonModelSerializer = configuration_registry.person_model_serializer
PersonRepresentationSerializer = configuration_registry.person_representation_serializer

from .companies import (
    BankModelSerializer,
    CompanyModelListSerializer,
    CompanyModelSerializer,
)
from .contacts import (
    AddressContactRepresentationSerializer,
    AddressContactSerializer,
    BankingContactRepresentationSerializer,
    BankingContactSerializer,
    ReadOnlyBankingContactSerializer,
    CityRepresentationSerializer,
    EmailContactRepresentationSerializer,
    EmailContactSerializer,
    SocialMediaContactRepresentationSerializer,
    SocialMediaContactSerializer,
    TelephoneContactRepresentationSerializer,
    TelephoneContactSerializer,
    WebsiteContactRepresentationSerializer,
    WebsiteContactSerializer,
)
from .entries import (
    CompanyRepresentationSerializer,
    CompanyTypeModelSerializer,
    CompanyTypeRepresentationSerializer,
    CustomerStatusModelSerializer,
    CustomerStatusRepresentationSerializer,
    EntryModelSerializer,
    EntryRepresentationSerializer,
    InternalUserProfileRepresentationSerializer,
    PersonRepresentationSerializer,
    SpecializationModelSerializer,
    SpecializationRepresentationSerializer,
)
from .entry_representations import (
    EntryRepresentationSerializer,
    EntryUnlinkedRepresentationSerializer,
)
from .persons import (
    NewPersonModelSerializer,
    PersonModelListSerializer,
    PersonModelSerializer,
)
from .relationships import (
    ClientManagerModelSerializer,
    ClientManagerRelationshipRepresentationSerializer,
    EmployerEmployeeRelationshipSerializer,
    PositionModelSerializer,
    PositionRepresentationSerializer,
    RelationshipModelSerializer,
    RelationshipRepresentationSerializer,
    RelationshipTypeModelSerializer,
    RelationshipTypeRepresentationSerializer,
    UserIsClientModelSerializer,
)



import pytest
from rest_framework import status

from wbcore.contrib.geography.factories import ContinentFactory
from wbcore.contrib.geography.models import Geography
from wbcore.contrib.geography.serializers import GeographyModelSerializer
from wbcore.contrib.geography.viewsets import (
    GeographyModelViewSet,
    GeographyRepresentationViewSet,
)


@pytest.mark.django_db
@pytest.mark.with_db
class TestViewsets:
    @pytest.fixture
    def continents(self):
        return ContinentFactory.create_batch(3)

    @pytest.fixture
    def continent(self, continents):
        return continents[0]

    @pytest.fixture
    def continent_build(self):
        instance = ContinentFactory.build()
        return GeographyModelSerializer(instance).data

    @pytest.mark.parametrize("viewset", [GeographyModelViewSet, GeographyRepresentationViewSet])
    def test_get(self, viewset, request_factory, super_user, continents):
        # Arrange
        request = request_factory.get("")
        request.user = super_user
        viewset = viewset.as_view({"get": "list"})
        # Act
        response = viewset(request)
        # Assert
        assert len(response.data["results"]) == 3
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("viewset", [GeographyModelViewSet, GeographyRepresentationViewSet])
    def test_retrieve(self, viewset, request_factory, super_user, continent):
        # Arrange
        request = request_factory.get("")
        request.user = super_user
        viewset = viewset.as_view({"get": "retrieve"})
        # Act
        response = viewset(request, pk=continent.id)
        instance = response.data.get("instance")
        # Assert
        assert instance is not None
        assert instance["id"] == continent.id
        assert response.status_code == status.HTTP_200_OK

    def test_create(self, request_factory, super_user, continent_build):
        # Arrange
        request = request_factory.post("", data=continent_build, format="json")
        request.user = super_user
        viewset = GeographyModelViewSet.as_view({"post": "create"})
        # Act
        response = viewset(request)
        # Assert
        assert response.status_code == status.HTTP_201_CREATED

    def test_delete(self, request_factory, super_user, continents):
        # Arrange
        entry_id = continents[1].id
        request = request_factory.delete("", args=entry_id)
        request.user = super_user
        viewset = GeographyModelViewSet.as_view({"delete": "destroy"})
        # Act
        response = viewset(request, pk=entry_id)
        # Assert
        assert response.status_code == status.HTTP_204_NO_CONTENT
        assert Geography.objects.count() == 2
        assert not Geography.objects.filter(id=entry_id).exists()

    def test_put(self, request_factory, super_user, continent, continent_build):
        # Arrange
        continent_build["id"] = continent.id
        request = request_factory.put("", data=continent_build, format="json")
        request.user = super_user
        viewset = GeographyModelViewSet.as_view({"put": "update"})
        # Act
        response = viewset(request, pk=continent.id)
        # Assert
        assert response.status_code == status.HTTP_200_OK

    def test_patch(self, request_factory, super_user, continent):
        # Arrange
        new_field_data = "Foo Bar"
        request = request_factory.patch("", data={"name": new_field_data})
        request.user = super_user
        viewset = GeographyModelViewSet.as_view({"patch": "partial_update"})
        # Act
        response = viewset(request, pk=continent.id)
        continent.refresh_from_db()
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert getattr(continent, "name") == new_field_data



from rest_framework.reverse import reverse

from wbcore.metadata.configs.endpoints import EndpointViewConfig


class EntryModelEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return reverse("wbcore:directory:entry-list", args=[self.view.kwargs["entry_id"]], request=self.request)


class CompanyModelEndpointConfig(EndpointViewConfig):
    def get_create_endpoint(self, **kwargs):
        return reverse("wbcore:directory:company-list", args=[], request=self.request)


class PersonModelEndpointConfig(EndpointViewConfig):
    def get_create_endpoint(self, **kwargs):
        return reverse("wbcore:directory:person-list", args=[], request=self.request)


class UserIsManagerEndpointConfig(EndpointViewConfig):
    def get_instance_endpoint(self, **kwargs):
        return "{{_additional_resources.detail}}"

    def _get_instance_endpoint(self):
        return self.get_instance_endpoint()

    def get_create_endpoint(self, **kwargs):
        base_url = "wbcore:directory:clientmanagerrelationship-list"
        filter_url = f"?relationship_manager={self.request.user.profile.id}"
        return f"{reverse(base_url, args=[], request=self.request)}{filter_url}"



from unittest.mock import patch

import pytest


@pytest.mark.django_db
class TestSplitStep:
    @patch("wbcore.contrib.workflow.models.step.activate_step.delay")
    @patch("wbcore.contrib.workflow.models.step.Step.get_all_valid_outgoing_transitions")
    @patch("wbcore.contrib.workflow.models.step.Step.set_finished")
    def test_run(
        self,
        mock_finished,
        mock_transitions,
        mock_activate,
        process_step_factory,
        transition_factory,
        split_step_factory,
    ):
        transition1 = transition_factory()
        transition2 = transition_factory()
        transition_factory()
        mock_transitions.return_value = [transition1, transition2]
        step = split_step_factory()
        process_step = process_step_factory(step=step)
        step.run(process_step)
        assert mock_finished.call_args.args == (process_step,)
        assert mock_activate.call_args_list == [
            ((transition1.to_step.pk, process_step.process.pk),),
            ((transition2.to_step.pk, process_step.process.pk),),
        ]

    @patch("wbcore.contrib.workflow.models.step.activate_step.delay")
    @patch("wbcore.contrib.workflow.models.step.Step.get_all_valid_outgoing_transitions")
    @patch("wbcore.contrib.workflow.models.step.Step.set_failed")
    def test_run_failed(
        self,
        mock_failed,
        mock_transitions,
        mock_activate,
        process_step_factory,
        split_step_factory,
    ):
        mock_transitions.return_value = []
        step = split_step_factory()
        process_step = process_step_factory(step=step)
        step.run(process_step)
        assert mock_failed.call_args.args[0] == process_step
        assert not mock_activate.called



from django.utils.translation import gettext as _

from wbcore.metadata.configs.titles import TitleViewConfig


class StepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Steps")

    def get_create_title(self):
        return _("Create Step")

    def get_instance_title(self):
        return _("Step")


class UserStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("User Steps")

    def get_create_title(self):
        return _("Create User Step")

    def get_instance_title(self):
        return _("User Step")


class DecisionStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Decision Steps")

    def get_create_title(self):
        return _("Create Decision Step")

    def get_instance_title(self):
        return _("Decision Step")


class StartStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Start Steps")

    def get_create_title(self):
        return _("Create Start Step")

    def get_instance_title(self):
        return _("Start Step")


class SplitStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Split Steps")

    def get_create_title(self):
        return _("Create Split Step")

    def get_instance_title(self):
        return _("Split Step")


class JoinStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Join Steps")

    def get_create_title(self):
        return _("Create Join Step")

    def get_instance_title(self):
        return _("Join Step")


class ScriptStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Script Steps")

    def get_create_title(self):
        return _("Create Script Step")

    def get_instance_title(self):
        return _("Script Step")


class EmailStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Email Steps")

    def get_create_title(self):
        return _("Create Email Step")

    def get_instance_title(self):
        return _("Email Step")


class FinishStepTitleConfig(TitleViewConfig):
    def get_list_title(self):
        return _("Finish Steps")

    def get_create_title(self):
        return _("Create Finish Step")

    def get_instance_title(self):
        return _("Finish Step")



from django.utils.translation import gettext as _

from wbcore.menus import Menu, MenuItem

TEAM_MENUITEM = MenuItem(
    label=_("Teams"),
    endpoint="example_app:team-list",
    add=MenuItem(
        label=_("Create Team"),
        endpoint="example_app:team-list",
    ),
)
LEAGUE_MENUITEM = MenuItem(
    label=_("Leagues"),
    endpoint="example_app:league-list",
    add=MenuItem(
        label=_("Create League"),
        endpoint="example_app:league-list",
    ),
)
STADIUM_MENUITEM = MenuItem(
    label=_("Stadiums"),
    endpoint="example_app:stadium-list",
    add=MenuItem(
        label=_("Create Stadium"),
        endpoint="example_app:stadium-list",
    ),
)
PERSONS_MENUITEM = MenuItem(
    label=_("Persons"),
    endpoint="example_app:person-list",
    add=MenuItem(
        label=_("Create Person"),
        endpoint="example_app:person-list",
    ),
)
SPORT_MENUITEM = MenuItem(
    label=_("Sports"),
    endpoint="example_app:sport-list",
    add=MenuItem(
        label=_("Create Sport"),
        endpoint="example_app:sport-list",
    ),
)
SEASON_MENUITEM = MenuItem(
    label=_("Season"),
    endpoint="example_app:season-list",
    add=MenuItem(label=_("Create Season"), endpoint="example_app:season-list"),
)
PLAYER_MENUITEM = MenuItem(
    label=_("Players"),
    endpoint="example_app:player-list",
    add=MenuItem(
        label=_("Create Player"),
        endpoint="example_app:player-list",
    ),
)
ROLE_MENUITEM = MenuItem(
    label=_("Roles"),
    endpoint="example_app:role-list",
    add=MenuItem(
        label=_("Create Role"),
        endpoint="example_app:role-list",
    ),
)
MATCH_MENUITEM = MenuItem(
    label=_("Matches"),
    endpoint="example_app:match-list",
    add=MenuItem(
        label=_("Create Match"),
        endpoint="example_app:match-list",
    ),
)
EVENT_MENUITEM = MenuItem(
    label=_("Events"),
    endpoint="example_app:event-list",
    add=MenuItem(
        label=_("Create Event"),
        endpoint="example_app:event-list",
    ),
)
EVENTTYPE_MENUITEM = MenuItem(
    label=_("Event Types"),
    endpoint="example_app:eventtype-list",
    add=MenuItem(
        label=_("Create Event Type"),
        endpoint="example_app:eventtype-list",
    ),
)

EXAMPLE_APP_MENU = Menu(
    label="Sports App",
    items=[
        Menu(label=_("Persons"), items=[PERSONS_MENUITEM, PLAYER_MENUITEM]),
        ROLE_MENUITEM,
        TEAM_MENUITEM,
        STADIUM_MENUITEM,
        Menu(label=_("Leagues"), items=[LEAGUE_MENUITEM, SEASON_MENUITEM]),
        SPORT_MENUITEM,
        MATCH_MENUITEM,
        Menu(label=_("Events"), items=[EVENT_MENUITEM, EVENTTYPE_MENUITEM]),
    ],
)



import random

import factory

from wbcore.contrib.example_app.models import Event, EventType
from wbcore.contrib.icons.icons import WBIcon


class EventTypeFactory(factory.django.DjangoModelFactory):
    name = factory.Sequence(lambda n: "Event %d" % n)
    sport = factory.SubFactory("wbcore.contrib.example_app.factories.SportFactory")
    points = factory.Faker("pyint", min_value=0, max_value=3)
    icon = factory.LazyAttribute(lambda x: WBIcon[random.choice(WBIcon.names)].icon)
    color = factory.Faker("color")

    class Meta:
        model = EventType


class EventFactory(factory.django.DjangoModelFactory):
    person = factory.SubFactory("wbcore.contrib.example_app.factories.SportPersonFactory")
    match = factory.SubFactory("wbcore.contrib.example_app.factories.MatchFactory")
    minute = factory.Faker("random_int", min=1, max=factory.SelfAttribute("..match.league.sport.match_duration"))
    event_type = factory.SubFactory(EventTypeFactory, sport=factory.SelfAttribute("..match.sport"))

    class Meta:
        model = Event



# Generated by Django 4.1 on 2023-03-30 09:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0001_initial"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="notificationusertoken",
            constraint=models.UniqueConstraint(
                fields=("user", "token", "device_type"), name="unique_user_token_device"
            ),
        ),
    ]



from wbcore.contrib.currency.models import Currency
from wbcore.metadata.configs.titles import TitleViewConfig


class CurrencyFXRatesCurrencyTitleConfig(TitleViewConfig):
    def get_list_title(self):
        currency = Currency.objects.get(id=self.view.kwargs["currency_id"])
        return f"Rates for {currency.key} ({currency.symbol})"



import factory
from django.db.models import signals

from wbcore.contrib.workflow.models import Condition


@factory.django.mute_signals(signals.post_save)
class ConditionFactory(factory.django.DjangoModelFactory):
    attribute_name = factory.SelfAttribute("transition.to_step.workflow.status_field")
    expected_value = factory.Faker("text", max_nb_chars=64)
    transition = factory.SubFactory("wbcore.contrib.workflow.factories.TransitionFactory")
    operator = factory.Iterator(Condition.Operator.values)
    negate_operator = factory.Faker("pybool")

    class Meta:
        model = Condition



from django.utils.translation import gettext_lazy as _

from wbcore import filters
from wbcore.contrib.example_app.models import League, Sport


class SportFilter(filters.FilterSet):
    leagues = filters.ModelChoiceFilter(
        label=_("Associated League"),
        queryset=League.objects.all(),
        endpoint=League.get_representation_endpoint(),
        value_key=League.get_representation_value_key(),
        label_key=League.get_representation_label_key(),
    )
    match_duration__gte = filters.NumberFilter(
        label=_("Match Duration"), lookup_expr="gte", field_name="match_duration"
    )
    match_duration__lte = filters.NumberFilter(
        label=_("Match Duration"), lookup_expr="lte", field_name="match_duration"
    )

    class Meta:
        model = Sport
        fields = {
            "name": ["exact", "icontains"],
            "rules": ["exact", "icontains"],
        }



from typing import Generator

from colorfield.fields import ColorField
from django.db import models
from django_better_admin_arrayfield.models.fields import ArrayField


class ColorGradient(models.Model):
    """
    Utility class for color gradient support in report
    """

    title = models.CharField(max_length=128, default="")
    colors = ArrayField(ColorField(default="#FF0000"), default=list)

    def __str__(self) -> str:
        return self.title

    def get_gradient(self, color: str | None = None, color_size: int = -1) -> Generator:
        """
        Return the consecutive colors (gradient) given a color based on the closest distance

        Args:
            color: a hex color. Optional.
            color_size: The number of colors the list should return. Optional.

        Returns:
            A generator of colors bounds in size by the color_size parameters or the gradient defaults color palette size
        """

        def _hex2rgb(_color):
            _color = _color.replace("#", "")
            return tuple(int(_color[i : i + 2], 16) for i in (0, 2, 4))

        if color_size < 0:
            color_size = len(self.colors)

        index = 0
        if color:
            color_rgb = _hex2rgb(color)
            nearest_color = min(
                self.colors, key=lambda subject: sum((s - q) ** 2 for s, q in zip(_hex2rgb(subject), color_rgb))
            )

            index = self.colors.index(nearest_color)
        for i in range(color_size):
            yield self.colors[(index + i) % len(self.colors)]


class TransparencyMixin:
    def get_cell_formatting(self, request):
        return [
            {
                "column": None,
                "conditions": [
                    {
                        "condition": None,
                        "style": {"backgroundColor": f'rgba(255, 255, 255, {getattr(self, "TRANSPARENCY", 0.4)})'},
                    },
                ],
            }
        ]



import os
from datetime import timedelta
from unittest.mock import patch

import pytest
from django.core import mail
from django.core.validators import URLValidator
from django.utils import timezone
from slugify import slugify
from wbcore.contrib.documents.models.documents import (
    Document,
    send_email_as_task,
    upload_to,
)


@pytest.mark.django_db
@pytest.mark.with_db
class TestDocumentModel:
    @pytest.fixture
    def email(self):
        return "foo@bar.com"

    def test_init(self, document):
        assert document.id is not None

    def test_upload_to(self, document, document_type_factory):
        parent_document_type = document_type_factory.create()
        document_type = document_type_factory.create(parent=parent_document_type)
        document.document_type = document_type
        document.save()
        assert (
            upload_to(document, "foo")
            == f"dms/document/{slugify(parent_document_type.name)}/{slugify(document_type.name)}/foo"
        )

    @pytest.mark.parametrize("document__file", ["/foo/bar"])
    def test_filename(self, document):
        assert document.filename == os.path.basename(document.file.name)

    def test_linked_objects(self, document_factory):
        document = document_factory.create()
        link_obj = document_factory.create()
        document.link(link_obj)
        assert list(document.linked_objects)[0] == link_obj

    @pytest.mark.parametrize(
        "sharing_seconds_duration, one_time_link",
        [(0, False), (1, False), (0, True), (1, True)],
    )
    def test_generate_shareable_link(self, document, sharing_seconds_duration, one_time_link):
        document.generate_shareable_link(
            sharing_seconds_duration=sharing_seconds_duration,
            one_time_link=one_time_link,
        )
        assert (
            document.shareable_links.filter(
                valid_until__isnull=sharing_seconds_duration == 0,
                one_time_link=one_time_link,
            ).count()
            == 1
        )

    @patch("wbcore.contrib.documents.models.documents.send_email_as_task.delay")
    def test_send_email(self, mock_fct, document, email):
        document.send_email(email)
        assert mock_fct.call_count == 1

    def test_get_for_object(self, document_factory):
        document = document_factory.create()
        link_obj = document_factory.create()
        document.link(link_obj)
        assert Document.get_for_object(link_obj).first() == document

    @pytest.mark.parametrize(
        "to_emails, as_link, subject, from_email, body",
        [
            (email, False, None, None, None),
            (email, False, "Some paragraph", email, "A different paragraph"),
            (email, True, None, None, None),
        ],
    )
    def test_send_email_as_task(self, document, to_emails, as_link, subject, from_email, body):
        assert len(mail.outbox) == 0
        send_email_as_task(
            document.id,
            to_emails,
            as_link=as_link,
            subject=subject,
            from_email=from_email,
            body=body,
        )
        assert len(mail.outbox) == 1
        msg = mail.outbox[0]
        if as_link:
            assert document.shareable_links.count() == 1
            assert len(msg.attachments) == 0
            assert len(msg.alternatives) == 1
        else:
            assert document.shareable_links.count() == 0
            assert len(msg.attachments) == 1
            assert len(msg.alternatives) == 2
        assert msg


@pytest.mark.django_db
class TestDocumentTypeModel:
    def test_init(self, document_type):
        assert document_type.id is not None

    def test_str(self, document_type_factory):
        document_type = document_type_factory.create(parent=document_type_factory.create())
        document_type.save()
        assert str(document_type) == document_type.compute_str()


@pytest.mark.django_db
class TestShareableLinkModel:
    @pytest.mark.parametrize(
        "shareable_link__manual_invalid, shareable_link__valid_until, shareable_link__one_time_link",
        [
            (True, None, False),
            (False, timezone.now() - timedelta(seconds=60), False),
            (False, None, True),
        ],
    )
    def test_unvalid_link(self, shareable_link):
        if shareable_link.one_time_link:
            assert shareable_link.is_valid()
            shareable_link.access()
        assert not shareable_link.is_valid()

    def test_link(self, shareable_link):
        URLValidator()(shareable_link.link)

    def test_access(self, shareable_link):
        assert shareable_link.accesses.count() == 0
        shareable_link.access()
        assert shareable_link.accesses.count() == 1

    def test_init(self, shareable_link):
        assert shareable_link.id is not None


@pytest.mark.django_db
class TestShareableLinkAccessModel:
    def test_init(self, shareable_link_access):
        assert shareable_link_access.id is not None



from typing import Optional

from django.utils.translation import gettext as _

from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display import DisplayViewConfig
from wbcore.metadata.configs.display.instance_display import (
    Display,
    Inline,
    Layout,
    Page,
    Section,
    Style,
)
from wbcore.metadata.configs.display.instance_display.operators import default


class WorkflowDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=(
                dp.Field(key="name", label=_("Name")),
                dp.Field(key="single_instance_execution", label=_("Single Instance Execution")),
                dp.Field(key="model", label=_("Attached Model")),
                dp.Field(key="status_field", label=_("Model Status Field")),
                dp.Field(key="preserve_instance", label=_("Preserve Instance")),
            ),
        )

    def get_instance_display(self) -> Display:
        model_section = Section(
            key="model_section",
            collapsible=False,
            title=_("Attached Model"),
            display=Display(
                pages=[
                    Page(
                        title=_("Model"),
                        layouts={
                            default(): Layout(
                                grid_template_areas=[["model", "status_field", "preserve_instance"]],
                                grid_template_columns=[
                                    "minmax(min-content, 2fr)",
                                    "minmax(min-content, 2fr)",
                                    "minmax(min-content, 1.5fr)",
                                    "minmax(min-content, 1.5fr)",
                                ],
                                grid_auto_rows=Style.MIN_CONTENT,
                            )
                        },
                    ),
                ]
            ),
        )
        steps_section = Section(
            key="steps_section",
            collapsible=True,
            title=_("Steps"),
            display=Display(
                pages=[
                    Page(
                        title=_("Steps"),
                        layouts={
                            default(): Layout(
                                grid_template_areas=[["steps_inline"]],
                                grid_auto_columns="minmax(min-content, 1fr)",
                                grid_auto_rows=Style.MIN_CONTENT,
                                inlines=[Inline(key="steps_inline", endpoint="steps")],
                            )
                        },
                    ),
                ]
            ),
        )
        transitions_section = Section(
            key="transitions_section",
            collapsible=True,
            title=_("Transitions"),
            display=Display(
                pages=[
                    Page(
                        title=_("Transitions"),
                        layouts={
                            default(): Layout(
                                grid_template_areas=[["transitions_inline"]],
                                grid_auto_columns="minmax(min-content, 1fr)",
                                grid_auto_rows=Style.MIN_CONTENT,
                                inlines=[Inline(key="transitions_inline", endpoint="transitions")],
                            )
                        },
                    ),
                ]
            ),
        )
        processes_section = Section(
            key="processes_section",
            collapsible=True,
            title=_("Processes"),
            display=Display(
                pages=[
                    Page(
                        title=_("Processes"),
                        layouts={
                            default(): Layout(
                                grid_template_areas=[["processes_inline"]],
                                grid_auto_columns="minmax(min-content, 1fr)",
                                grid_auto_rows=Style.MIN_CONTENT,
                                inlines=[Inline(key="processes_inline", endpoint="processes")],
                            )
                        },
                    ),
                ]
            ),
        )
        # data_section = Section(
        #     key="data_section",
        #     collapsible=True,
        #     title=_("Data"),
        #     display=Display(
        #         pages=[
        #             Page(
        #                 title=_("Data"),
        #                 layouts={
        #                     default(): Layout(
        #                         grid_template_areas=[["data_inline"]],
        #                         grid_auto_columns="minmax(min-content, 1fr)",
        #                         grid_auto_rows=Style.MIN_CONTENT,
        #                         inlines=[Inline(key="data_inline", endpoint="data")],
        #                     )
        #                 },
        #             ),
        #         ]
        #     ),
        # )
        grid_fields = [
            ["name", "single_instance_execution"],
            ["model_section", "."],
            ["graph", "graph"],
            ["steps_section", "steps_section"],
            ["transitions_section", "transitions_section"],
            ["processes_section", "processes_section"],
            # ["data_section", "data_section"],
        ]
        if "pk" not in self.view.kwargs:
            grid_fields = grid_fields[:2]

        return Display(
            pages=[
                Page(
                    title=_("Workflow"),
                    layouts={
                        default(): Layout(
                            grid_template_areas=grid_fields,
                            grid_template_columns=["minmax(min-content, 3fr)", "minmax(min-content, 1fr)"],
                            grid_auto_rows=Style.MIN_CONTENT,
                            column_gap=Style.rem(5),
                            sections=[
                                model_section,
                                steps_section,
                                processes_section,
                                # data_section,
                                transitions_section,
                            ],
                        ),
                    },
                ),
            ]
        )



import pytest
from django.contrib.contenttypes.models import ContentType
from rest_framework.test import APIRequestFactory

from wbcore.contrib.workflow.models import Step
from wbcore.contrib.workflow.viewsets import (
    EmailStepModelViewSet,
    StepModelViewSet,
    TransitionModelViewSet,
    WorkflowModelViewSet,
)
from wbcore.test.utils import get_or_create_superuser


@pytest.fixture()
def super_user_get_request():
    request = APIRequestFactory().get("")
    request.user = get_or_create_superuser()


@pytest.mark.django_db
class TestWorkflow:
    @pytest.fixture()
    def workflow_viewset(self):
        return WorkflowModelViewSet()

    def test_filter_attached_model(self, super_user_get_request, workflow_viewset, workflow_factory):
        workflow_factory()
        workflow_factory(model=ContentType.objects.first())
        qs = workflow_viewset.get_queryset()
        assert (
            workflow_viewset.filterset_class(request=super_user_get_request).filter_attached_model(qs, "", None) == qs
        )
        assert (
            workflow_viewset.filterset_class(request=super_user_get_request)
            .filter_attached_model(qs, "", "person")
            .count()
            == 1
        )
        assert (
            not workflow_viewset.filterset_class(request=super_user_get_request)
            .filter_attached_model(qs, "", "company")
            .exists()
        )

    def test_filter_by_data(self, super_user_get_request, workflow_viewset, workflow_factory, data_factory):
        workflow_factory()
        data = data_factory()
        qs = workflow_viewset.get_queryset()
        assert workflow_viewset.filterset_class(request=super_user_get_request).filter_by_data(qs, "", None) == qs
        assert (
            workflow_viewset.filterset_class(request=super_user_get_request).filter_by_data(qs, "", data).count() == 1
        )
        assert (
            workflow_viewset.filterset_class(request=super_user_get_request).filter_by_data(qs, "", data).first()
            == data.workflow
        )


@pytest.mark.django_db
class TestStep:
    def test_filter_by_transition(self, super_user_get_request, transition_factory, random_child_step_factory):
        transition = transition_factory()
        random_child_step_factory()
        mvs = StepModelViewSet(kwargs={})
        qs = mvs.get_queryset()
        assert mvs.get_filterset_class(request=super_user_get_request)().filter_by_transition(qs, "", None) == qs
        assert set(
            mvs.get_filterset_class(request=super_user_get_request)().filter_by_transition(qs, "", transition)
        ) == set(Step.objects.filter(id__in=[transition.from_step.pk, transition.to_step.pk]))

    def test_email_filter_template_name(self, super_user_get_request, email_step_factory):
        step = email_step_factory()
        email_step_factory()
        mvs = EmailStepModelViewSet(kwargs={})
        qs = mvs.get_queryset()
        assert mvs.get_filterset_class(request=super_user_get_request)().filter_template_name(qs, "", None) == qs
        assert (
            mvs.get_filterset_class(request=super_user_get_request)()
            .filter_template_name(qs, "", step.template.name)
            .count()
            == 1
        )
        assert (
            mvs.get_filterset_class(request=super_user_get_request)()
            .filter_template_name(qs, "", step.template.name)
            .first()
            == step
        )


@pytest.mark.django_db
class TestTransition:
    def test_filter_by_condition(self, super_user_get_request, condition_factory, transition_factory):
        transition_factory()
        condition = condition_factory()
        mvs = TransitionModelViewSet(kwargs={})
        qs = mvs.get_queryset()
        assert mvs.filterset_class(request=super_user_get_request).filter_by_condition(qs, "", None) == qs
        assert mvs.filterset_class(request=super_user_get_request).filter_by_condition(qs, "", condition).count() == 1
        assert (
            mvs.filterset_class(request=super_user_get_request).filter_by_condition(qs, "", condition).first()
            == condition.transition
        )



# Generated by Django 4.1.8 on 2023-04-06 14:14

import django.db.models.deletion
import wbcore.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="TagGroup",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("managed", models.BooleanField(default=False)),
                ("title", models.CharField(max_length=255, unique=True)),
            ],
            options={
                "verbose_name": "Tag Group",
                "verbose_name_plural": "Tag Groups",
            },
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
                ),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("managed", models.BooleanField(default=False)),
                ("title", models.CharField(max_length=255)),
                ("slug", models.CharField(blank=True, max_length=255, null=True)),
                ("color", wbcore.contrib.color.fields.ColorField(default="#D3D3D3")),
                ("description", models.TextField(blank=True, default="")),
                (
                    "content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="associated_tags",
                        to="contenttypes.contenttype",
                    ),
                ),
                ("groups", models.ManyToManyField(blank=True, to="tags.taggroup")),
            ],
            options={
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
                "unique_together": {("slug", "content_type")},
            },
        ),
    ]



from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy
from rest_framework.reverse import reverse

from wbcore.contrib.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig


class EntryModelButtonConfig(ButtonViewConfig):
    CRM_BUTTONS = ()

    CONTACT_BUTTONS = (
        bt.WidgetButton(key="relationships", label=gettext_lazy("Relationships"), icon=WBIcon.GROUPS.icon),
        bt.WidgetButton(key="emails", label=gettext_lazy("Emails"), icon=WBIcon.MAIL.icon),
        bt.WidgetButton(key="telephones", label=gettext_lazy("Telephones"), icon=WBIcon.PHONE.icon),
        bt.WidgetButton(key="addresses", label=gettext_lazy("Addresses"), icon=WBIcon.LOCATION.icon),
        bt.WidgetButton(key="websites", label=gettext_lazy("Websites"), icon=WBIcon.PUBLIC.icon),
        bt.WidgetButton(key="bankings", label=gettext_lazy("Bankings"), icon=WBIcon.BANK.icon),
        bt.WidgetButton(key="social_media", label=gettext_lazy("Social Media"), icon=WBIcon.NEWSPAPER.icon),
    )

    def get_custom_list_instance_buttons(self):
        return {
            bt.DropDownButton(
                label=_("Quick Action"),
                icon=WBIcon.UNFOLD.icon,
                buttons=(
                    bt.WidgetButton(key="employees", label=_("Employees"), icon=WBIcon.PEOPLE.icon, weight=2),
                    bt.WidgetButton(
                        key="manager", label=_("Relationship Managers"), icon=WBIcon.SUPERVISE.icon, weight=3
                    ),
                ),
                weight=1,
            )
        }

    def get_custom_instance_buttons(self):
        return {
            bt.WidgetButton(key="employees", label=_("Employees"), icon=WBIcon.PEOPLE.icon, weight=2),
            bt.WidgetButton(key="manager", label=_("Relationship Managers"), icon=WBIcon.SUPERVISE.icon, weight=3),
            bt.DropDownButton(
                label=_("More Information"), icon=WBIcon.UNFOLD.icon, buttons=self.CONTACT_BUTTONS, weight=4
            ),
        }


class PersonModelButtonConfig(EntryModelButtonConfig):
    CRM_BUTTONS = EntryModelButtonConfig.CRM_BUTTONS + (
        bt.WidgetButton(key="client", label=gettext_lazy("Clients"), icon=WBIcon.PEOPLE.icon),
        bt.WidgetButton(
            key="manager", label=gettext_lazy("Relationship Managers"), icon=WBIcon.SUPERVISE.icon, weight=3
        ),
    )

    def get_custom_instance_buttons(self):
        return {
            bt.WidgetButton(key="employers", label=_("Employers"), icon=WBIcon.WORK.icon, weight=2),
            bt.DropDownButton(label=_("Client/Manager"), icon=WBIcon.UNFOLD.icon, buttons=self.CRM_BUTTONS, weight=3),
            bt.DropDownButton(
                label=_("More Information"), icon=WBIcon.UNFOLD.icon, buttons=self.CONTACT_BUTTONS, weight=4
            ),
        }


class CompanyModelButtonConfig(EntryModelButtonConfig):
    CRM_BUTTONS = EntryModelButtonConfig.CRM_BUTTONS + (
        bt.WidgetButton(key="employees", label=gettext_lazy("Employees"), icon=WBIcon.PEOPLE.icon),
    )


class EmployeeCompanyModelButtonConfig(ButtonViewConfig):
    def get_custom_buttons(self):
        if not self.view.kwargs.get("pk", None):
            base_url = reverse("wbcore:directory:person-list", args=[], request=self.request)
            return {
                bt.WidgetButton(
                    endpoint=f"{base_url}?employers={self.view.kwargs.get('company_id')}",
                    label=_("New Person"),
                    icon=WBIcon.PERSON_ADD.icon,
                    new_mode=True,
                )
            }
        return {}

    def get_custom_list_instance_buttons(self):
        return {
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("directory:employeremployee",),
                key="delete",
                label=_("Delete Relationship"),
                icon=WBIcon.DELETE.icon,
                description_fields=_("<p> Are you sure you want to delete the relationship? </p>"),
                title=_("Delete Relationship"),
                action_label=_("Deletion"),
            ),
            bt.WidgetButton(
                key="eer_relationship_instance", label=_("Relationship Instance"), icon=WBIcon.GROUPS.icon
            ),
        }


class EmployerPersonModelButtonConfig(ButtonViewConfig):
    def get_custom_buttons(self):
        if not self.view.kwargs.get("pk", None):
            base_url = reverse("wbcore:directory:company-list", args=[], request=self.request)
            return {
                bt.WidgetButton(
                    endpoint=f"{base_url}?employees={self.view.kwargs.get('person_id')}",
                    label=_("New Company"),
                    icon=WBIcon.ADD.icon,
                    new_mode=True,
                )
            }
        return {}

    def get_custom_list_instance_buttons(self):
        return {
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("directory:employeremployee",),
                key="delete",
                label=_("Delete Relationship"),
                icon=WBIcon.DELETE.icon,
                description_fields=_("<p> Are you sure you want to delete the relationship? </p>"),
                title=_("Delete Relationship"),
                action_label=_("Deletion"),
            ),
            bt.WidgetButton(
                key="eer_relationship_instance", label=_("Relationship Instance"), icon=WBIcon.GROUPS.icon
            ),
        }



import pytest
from pytest_mock import MockerFixture
from rest_framework.exceptions import ValidationError
from schwifty import IBAN

from wbcore.contrib.directory.models import Entry, RelationshipType
from wbcore.contrib.directory.serializers import (
    AddressContactSerializer,
    BankingContactSerializer,
    ClientManagerModelSerializer,
    EmailContactSerializer,
    RelationshipTypeModelSerializer,
    RelationshipTypeRepresentationSerializer,
    TelephoneContactSerializer,
    WebsiteContactSerializer,
)

from ..factories import ClientManagerRelationshipFactory, PersonFactory


@pytest.mark.serializer_tests
class TestRelationshipTypeSerializers:
    @pytest.fixture
    def relationship_type(self, mocker: MockerFixture):
        relationship_type = RelationshipType()
        mocker.patch.object(relationship_type, "id", 1)
        mocker.patch.object(relationship_type, "title", "Test Type")
        return relationship_type

    @pytest.fixture
    def expected_data(self, relationship_type):
        return {"id": relationship_type.id, "title": relationship_type.title, "counter_relationship": None}

    def test_relationship_type_repr_serializer(self, mocker: MockerFixture, relationship_type, expected_data):
        serializer = RelationshipTypeRepresentationSerializer(instance=relationship_type)
        assert serializer.data == expected_data

    def test_relationship_type_model_serializer(self, mocker: MockerFixture, relationship_type, expected_data):
        serializer = RelationshipTypeModelSerializer(instance=relationship_type)
        expected_data["_counter_relationship"] = None
        assert serializer.data == expected_data


@pytest.mark.serializer_tests
@pytest.mark.with_db
@pytest.mark.django_db
class TestClientManagerSerializersNew:
    @pytest.fixture
    def manager(self):
        return PersonFactory()

    @pytest.fixture
    def client(self):
        return PersonFactory()

    @pytest.fixture
    def primary_cmr(self, manager, client):
        return ClientManagerRelationshipFactory(client=client, relationship_manager=manager, primary=True)

    def test_create_new_request(self, manager, client):
        """
        Validate method should not fail when given data for a new relationship.
        """
        data = {"client": client, "relationship_manager": manager}
        validated_data = ClientManagerModelSerializer().validate(data)
        assert validated_data == data

    def test_create_existing_request(self, primary_cmr):
        """
        Validate method should fail when given data for an already existing client manager relationship.
        """
        manager = primary_cmr.relationship_manager
        client = primary_cmr.client
        error_message = f"{manager} is already in charge of {client}."
        serializer = ClientManagerModelSerializer()
        data = {"client": client, "relationship_manager": manager, "primary": True}
        with pytest.raises(ValidationError) as exc_info:
            serializer.validate(data)
        assert exc_info.value.args[0]["non_field_errors"] == error_message

    def test_self_relationship_managers(self, manager):
        """
        Validate method should fail when client and manager are the same person.
        """
        serializer = ClientManagerModelSerializer()
        data = {"client": manager, "relationship_manager": manager}
        error_message = "Client and relationship manager cannot be the same person."
        with pytest.raises(ValidationError) as exc_info:
            serializer.validate(data)
        assert exc_info.value.args[0]["relationship_manager"] == error_message

    def test_degrade_primary_manager(self, primary_cmr):
        """
        Validate method should fail when trying to make primary manager non-primary.
        """
        manager = primary_cmr.relationship_manager
        client = primary_cmr.client
        serializer = ClientManagerModelSerializer()
        error_message = "Cannot degrade primary manager. Make a primary request for a different manager instead."
        data = {"client": client, "relationship_manager": manager, "primary": False}
        with pytest.raises(ValidationError) as exc_info:
            serializer.validate(data)
        assert exc_info.value.args[0]["primary"] == error_message


@pytest.mark.serializer_tests
class TestContactSerializersValidation:
    @pytest.fixture
    def iban(self):
        return IBAN("AD1400080001001234567890")

    @pytest.fixture
    def contact_data(iban, mocker: MockerFixture):
        return {
            "AddressContact": {
                "serializer": AddressContactSerializer(),
                "data": {
                    "entry": mocker.MagicMock(spec=Entry),
                    "street": "Foo Str",
                    "street_additional": 5,
                    "zip": 22222,
                    "geography_city": "Bar City",
                },
                "error_message": "Address already in use",
                "key": "non_field_errors",
            },
            "BankingContact": {
                "serializer": BankingContactSerializer(),
                "data": {"iban": IBAN("AD1400080001001234567890"), "entry": mocker.MagicMock(spec=Entry)},
                "error_message": "IBAN already in use",
                "key": "iban",
            },
            "EmailContact": {
                "serializer": EmailContactSerializer(),
                "data": {"address": "foo@bar.com", "entry": mocker.MagicMock(spec=Entry)},
                "error_message": "E-Mail address already in use",
                "key": "address",
            },
            "TelephoneContact": {
                "serializer": TelephoneContactSerializer(),
                "data": {"number": 12345, "entry": mocker.MagicMock(spec=Entry)},
                "error_message": "Phone number already in use",
                "key": "number",
            },
            "WebsiteContact": {
                "serializer": WebsiteContactSerializer(),
                "data": {"url": "www.foo.bar", "entry": mocker.MagicMock(spec=Entry)},
                "error_message": "Website already in use",
                "key": "url",
            },
        }

    def _mock_qs(self, mocker: MockerFixture, path: str, exists=False) -> None:
        mock_filter = mocker.patch(f"{path}.objects.filter")
        mock_queryset = mocker.MagicMock()
        mock_filter.return_value = mock_queryset
        mock_queryset.exists.return_value = exists

    def test_create_invalid_email(self, mocker: MockerFixture):
        mock_path = "wbcore.contrib.directory.models.EmailContact"
        self._mock_qs(mocker, mock_path, False)
        serializer = EmailContactSerializer()
        data = {"address": "foobar.com"}
        with pytest.raises(ValidationError) as exc_info:
            serializer.validate(data)
        assert exc_info.value.args[0]["address"] == "Invalid e-mail address"

    def test_create_invalid_iban(self, mocker: MockerFixture):
        mock_path = "wbcore.contrib.directory.models.BankingContact"
        self._mock_qs(mocker, mock_path)
        serializer = BankingContactSerializer()
        data = {"entry": mocker.MagicMock(spec=Entry), "iban": "Invalid Iban"}
        with pytest.raises(ValidationError):
            serializer.validate(data)

    def test_create_invalid_bic(self, mocker: MockerFixture, iban):
        mock_path = "wbcore.contrib.directory.models.BankingContact"
        self._mock_qs(mocker, mock_path)
        serializer = BankingContactSerializer()
        data = {"entry": mocker.MagicMock(spec=Entry), "iban": iban, "swift_bic": "Invalid Bic"}
        with pytest.raises(ValidationError):
            serializer.validate(data)

    def test_invalid_phone_number(self, mocker: MockerFixture):
        mock_path = "wbcore.contrib.directory.models.TelephoneContact"
        self._mock_qs(mocker, mock_path)
        serializer = TelephoneContactSerializer()
        data = {"entry": mocker.MagicMock(spec=Entry), "number": "003A56"}
        with pytest.raises(ValidationError) as exc_info:
            serializer.validate(data)
        assert exc_info.value.args[0]["number"] == "Invalid phone number format"

    @pytest.mark.parametrize(
        "contact", ["AddressContact", "BankingContact", "EmailContact", "TelephoneContact", "WebsiteContact"]
    )
    def test_create_invalid_contact(self, contact, contact_data, mocker: MockerFixture):
        data = contact_data[contact]
        mock_path = f"wbcore.contrib.directory.models.{contact}"
        self._mock_qs(mocker, mock_path, True)
        with pytest.raises(ValidationError) as exc_info:
            data["serializer"].validate(data["data"])
        assert data["error_message"] in exc_info.value.args[0][data["key"]]

    @pytest.mark.parametrize(
        "serializer",
        [
            AddressContactSerializer(),
            BankingContactSerializer(),
            EmailContactSerializer(),
            TelephoneContactSerializer(),
            WebsiteContactSerializer(),
        ],
    )
    def test_create_without_data(self, serializer):
        data = {}
        validated_data = serializer.validate(data)
        assert data == validated_data
